# Copyright 2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os

from awscli.compat import StringIO
from awscli.customizations.configure import (
    NOT_SET,
    ConfigValue,
    configure,
    profile_to_section,
)
from awscli.testutils import mock, unittest

from . import FakeSession


class TestConfigureCommand(unittest.TestCase):
    def setUp(self):
        self.writer = mock.Mock()
        self.global_args = mock.Mock()
        self.global_args.profile = None
        self.precanned = PrecannedPrompter(value='new_value')
        self.session = FakeSession({'config_file': 'myconfigfile'})
        self.configure = configure.ConfigureCommand(
            self.session, prompter=self.precanned, config_writer=self.writer
        )

    def assert_credentials_file_updated_with(self, new_values):
        called_args = self.writer.update_config.call_args_list
        credentials_file_call = called_args[0]
        expected_creds_file = os.path.expanduser('~/fake_credentials_filename')
        self.assertEqual(
            credentials_file_call,
            mock.call(
                new_values, expected_creds_file, check_permissions=True
            ),
        )

    def test_configure_command_sends_values_to_writer(self):
        self.configure(args=[], parsed_globals=self.global_args)
        # Credentials are always written to the shared credentials file.
        self.assert_credentials_file_updated_with(
            {
                'aws_access_key_id': 'new_value',
                'aws_secret_access_key': 'new_value',
            }
        )

        # Non-credentials config is written to the config file.
        self.writer.update_config.assert_called_with(
            {'region': 'new_value', 'output': 'new_value'}, 'myconfigfile'
        )

    def test_same_values_are_not_changed(self):
        # If the user enters the same value as the current value, we don't need
        # to write anything to the config.
        self.configure = configure.ConfigureCommand(
            self.session, prompter=EchoPrompter(), config_writer=self.writer
        )
        self.configure(args=[], parsed_globals=self.global_args)
        self.assertFalse(self.writer.update_config.called)

    def test_none_values_are_not_changed(self):
        # If a user hits enter, this will result in a None value which means
        # don't change the existing values.  In this case, we don't need
        # to write anything out to the config.
        user_presses_enter = None
        precanned = PrecannedPrompter(value=user_presses_enter)
        self.configure = configure.ConfigureCommand(
            self.session, prompter=precanned, config_writer=self.writer
        )
        self.configure(args=[], parsed_globals=self.global_args)
        self.assertFalse(self.writer.update_config.called)

    def test_create_configure_cmd_session_only(self):
        self.configure = configure.ConfigureCommand(self.session)
        self.assertIsInstance(self.configure, configure.ConfigureCommand)

    def test_some_values_changed(self):
        # Test the case where the user only wants to change a single_value.
        responses = {
            "AWS Access Key ID": None,
            "AWS Secret Access Key": None,
            "Default region name": None,
            "Default output format": "NEW OUTPUT FORMAT",
        }
        prompter = KeyValuePrompter(responses)
        self.configure = configure.ConfigureCommand(
            self.session, prompter=prompter, config_writer=self.writer
        )
        self.configure(args=[], parsed_globals=self.global_args)

        # We only need to write out the default output format.
        self.writer.update_config.assert_called_with(
            {'output': 'NEW OUTPUT FORMAT'}, 'myconfigfile'
        )

    def test_section_name_can_be_changed_for_profiles(self):
        # If the user specifies a profile we need to write this out to
        # the [profile myname] section.
        self.session.profile = 'myname'
        self.configure(args=[], parsed_globals=self.global_args)
        # Note the __section__ key name.
        self.assert_credentials_file_updated_with(
            {
                'aws_access_key_id': 'new_value',
                'aws_secret_access_key': 'new_value',
                '__section__': 'myname',
            }
        )
        self.writer.update_config.assert_called_with(
            {
                '__section__': 'profile myname',
                'region': 'new_value',
                'output': 'new_value',
            },
            'myconfigfile',
        )

    def test_session_says_profile_does_not_exist(self):
        # Whenever you try to get a config value from botocore,
        # it will raise an exception complaining about ProfileNotFound.
        # We should handle this case, and write out a new profile section
        # in the config file.
        session = FakeSession(
            {'config_file': 'myconfigfile'},
            profile_does_not_exist=True,
            profile='profile-does-not-exist',
        )
        self.configure = configure.ConfigureCommand(
            session, prompter=self.precanned, config_writer=self.writer
        )
        self.configure(args=[], parsed_globals=self.global_args)
        self.assert_credentials_file_updated_with(
            {
                'aws_access_key_id': 'new_value',
                'aws_secret_access_key': 'new_value',
                '__section__': 'profile-does-not-exist',
            }
        )
        self.writer.update_config.assert_called_with(
            {
                '__section__': 'profile profile-does-not-exist',
                'region': 'new_value',
                'output': 'new_value',
            },
            'myconfigfile',
        )

    def test_temporary_credentials_prompts_for_session_token(self):
        # When user enters temporary credentials (starting with ASIA),
        # should prompt for session token after secret key
        responses = {
            "AWS Access Key ID": "ASIATEMP123456789",
            "AWS Secret Access Key": "secret123",
            "AWS Session Token": "session_token_123",
            "Default region name": "us-west-2",
            "Default output format": "json",
        }
        prompter = KeyValuePrompter(responses)
        self.configure = configure.ConfigureCommand(
            self.session, prompter=prompter, config_writer=self.writer
        )
        self.configure(args=[], parsed_globals=self.global_args)

        # Should write all three credential values to credentials file
        self.assert_credentials_file_updated_with(
            {
                'aws_access_key_id': 'ASIATEMP123456789',
                'aws_secret_access_key': 'secret123',
                'aws_session_token': 'session_token_123',
            }
        )

        # Non-credentials config is written to the config file
        self.writer.update_config.assert_called_with(
            {'region': 'us-west-2', 'output': 'json'}, 'myconfigfile'
        )

    def test_regular_credentials_no_session_token_prompt(self):
        # When user enters regular credentials (not starting with ASIA),
        # should NOT prompt for session token
        responses = {
            "AWS Access Key ID": "AKIAREGULAR123456",
            "AWS Secret Access Key": "secret123",
            "Default region name": "us-west-2",
            "Default output format": "json",
        }
        prompter = KeyValuePrompter(responses)
        self.configure = configure.ConfigureCommand(
            self.session, prompter=prompter, config_writer=self.writer
        )
        self.configure(args=[], parsed_globals=self.global_args)

        # Should only write access key and secret key (no session token)
        self.assert_credentials_file_updated_with(
            {
                'aws_access_key_id': 'AKIAREGULAR123456',
                'aws_secret_access_key': 'secret123',
            }
        )

    def test_iam_user_credentials_remove_session_token(self):
        # When configuring IAM user credentials (AKIA), existing session token should be removed
        session = FakeSession({'config_file': 'myconfigfile'})
        session.config = {'aws_session_token': 'existing_token'}
        responses = {
            "AWS Access Key ID": "AKIAUSER123456789",
            "AWS Secret Access Key": "secret123",
            "Default region name": "us-west-2",
            "Default output format": "json",
        }
        prompter = KeyValuePrompter(responses)
        self.configure = configure.ConfigureCommand(
            session, prompter=prompter, config_writer=self.writer
        )
        self.configure(args=[], parsed_globals=self.global_args)

        # Should write credentials and remove session token (set to None)
        self.assert_credentials_file_updated_with(
            {
                'aws_access_key_id': 'AKIAUSER123456789',
                'aws_secret_access_key': 'secret123',
                'aws_session_token': None,
            }
        )

    def test_check_permissions_when_credential_values_provided(self):
        self.configure(args=[], parsed_globals=self.global_args)
        expected_creds_file = os.path.expanduser('~/fake_credentials_filename')
        self.writer.update_config.assert_any_call(
            mock.ANY, expected_creds_file, check_permissions=True
        )

    def test_no_check_permissions_when_no_credential_values_changed(self):
        user_presses_enter = None
        precanned = PrecannedPrompter(value=user_presses_enter)
        self.configure = configure.ConfigureCommand(
            self.session, prompter=precanned, config_writer=self.writer
        )
        self.configure(args=[], parsed_globals=self.global_args)
        self.writer.update_config.assert_not_called()


class TestInteractivePrompter(unittest.TestCase):
    def setUp(self):
        self.input_patch = mock.patch('awscli.compat.raw_input')
        self.mock_raw_input = self.input_patch.start()
        self.stdout = StringIO()
        self.stdout_patch = mock.patch('sys.stdout', self.stdout)
        self.stdout_patch.start()

    def tearDown(self):
        self.input_patch.stop()
        self.stdout_patch.stop()

    def test_access_key_is_masked(self):
        self.mock_raw_input.return_value = 'foo'
        prompter = configure.InteractivePrompter()
        response = prompter.get_value(
            current_value='myaccesskey',
            config_name='aws_access_key_id',
            prompt_text='Access key',
        )
        # First we should return the value from raw_input.
        self.assertEqual(response, 'foo')
        # We should also not display the entire access key.
        prompt_text = self.stdout.getvalue()
        self.assertNotIn('myaccesskey', prompt_text)
        self.assertRegex(prompt_text, r'\[\*\*\*\*.*\]')

    def test_access_key_not_masked_when_none(self):
        self.mock_raw_input.return_value = 'foo'
        prompter = configure.InteractivePrompter()
        response = prompter.get_value(
            current_value=None,
            config_name='aws_access_key_id',
            prompt_text='Access key',
        )
        # First we should return the value from raw_input.
        self.assertEqual(response, 'foo')
        prompt_text = self.stdout.getvalue()
        self.assertIn('[None]', prompt_text)

    def test_secret_key_is_masked(self):
        prompter = configure.InteractivePrompter()
        prompter.get_value(
            current_value='mysupersecretkey',
            config_name='aws_secret_access_key',
            prompt_text='Secret Key',
        )
        # We should also not display the entire secret key.
        prompt_text = self.stdout.getvalue()
        self.assertNotIn('mysupersecretkey', prompt_text)
        self.assertRegex(prompt_text, r'\[\*\*\*\*.*\]')

    def test_non_secret_keys_are_not_masked(self):
        prompter = configure.InteractivePrompter()
        prompter.get_value(
            current_value='mycurrentvalue',
            config_name='not_a_secret_key',
            prompt_text='Enter value',
        )
        # We should also not display the entire secret key.
        prompt_text = self.stdout.getvalue()
        self.assertIn('mycurrentvalue', prompt_text)
        self.assertRegex(prompt_text, r'\[mycurrentvalue\]')

    def test_session_token_is_masked(self):
        prompter = configure.InteractivePrompter()
        prompter.get_value(
            current_value='mysessiontoken123',
            config_name='aws_session_token',
            prompt_text='Session Token',
        )
        # Session token should be masked like other credentials
        prompt_text = self.stdout.getvalue()
        self.assertNotIn('mysessiontoken123', prompt_text)
        self.assertRegex(prompt_text, r'\[\*\*\*\*.*\]')

    def test_user_hits_enter_returns_none(self):
        # If a user hits enter, then raw_input returns the empty string.
        self.mock_raw_input.return_value = ''

        prompter = configure.InteractivePrompter()
        response = prompter.get_value(
            current_value=None,
            config_name='aws_access_key_id',
            prompt_text='Access key',
        )
        # We convert the empty string to None to indicate that there
        # was no input.
        self.assertIsNone(response)

    def test_compat_input_flushes_after_each_prompt(self):
        # Clear out the default patch
        self.stdout_patch.stop()

        # Create a mock stdout to record flush calls and replace stdout_patch
        self.stdout = mock.Mock()
        self.stdout_patch = mock.patch('sys.stdout', self.stdout)
        self.stdout_patch.start()

        # Make sure flush called at least once
        prompter = configure.InteractivePrompter()
        prompter.get_value(
            current_value='foo', config_name='bar', prompt_text='baz'
        )
        self.assertTrue(self.stdout.flush.called)

        # Make sure flush is called after *every* prompt
        self.stdout.reset_mock()
        prompter.get_value(
            current_value='foo2', config_name='bar2', prompt_text='baz2'
        )
        self.assertTrue(self.stdout.flush.called)


class TestConfigValueMasking(unittest.TestCase):
    def test_config_value_is_masked(self):
        config_value = ConfigValue(
            'fake_access_key', 'config_file', 'aws_access_key_id'
        )
        self.assertEqual(config_value.value, 'fake_access_key')
        config_value.mask_value()
        self.assertEqual(config_value.value, '****************_key')

    def test_dont_mask_unset_value(self):
        no_config = ConfigValue(NOT_SET, None, None)
        self.assertEqual(no_config.value, NOT_SET)
        no_config.mask_value()
        self.assertEqual(no_config.value, NOT_SET)


class TestProfileToSection(unittest.TestCase):
    def test_normal_profile(self):
        profile = 'my-profile'
        section = profile_to_section(profile)
        self.assertEqual('profile my-profile', section)

    def test_profile_with_spaces(self):
        profile = 'my spaced profile'
        section = profile_to_section(profile)
        self.assertEqual('profile \'my spaced profile\'', section)

    def test_profile_with_tab(self):
        profile = 'tab\ts'
        section = profile_to_section(profile)
        self.assertEqual('profile \'tab\ts\'', section)

    def test_profile_with_consecutive_spaces(self):
        profile = '    '
        section = profile_to_section(profile)
        self.assertEqual('profile \'    \'', section)

    def test_default_profile(self):
        profile = 'default'
        section = profile_to_section(profile)
        self.assertEqual(profile, section)


class PrecannedPrompter:
    def __init__(self, value):
        self._value = value

    def get_value(self, current_value, logical_name, prompt_text=''):
        return self._value


class EchoPrompter:
    def get_value(self, current_value, logical_name, prompt_text=''):
        return current_value


class KeyValuePrompter:
    def __init__(self, mapping):
        self.mapping = mapping

    def get_value(self, current_value, config_name, prompt_text=''):
        return self.mapping.get(prompt_text)



# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import json

from awscli.testutils import BaseAWSCommandParamsTest, BaseAWSHelpOutputTest


class TestListStreams(BaseAWSCommandParamsTest):
    prefix = ['kinesis', 'list-streams']

    def test_exclusive_start_stream_name_disables_auto_pagination(self):
        cmdline = self.prefix + ['--exclusive-start-stream-name', 'stream-1']
        self.parsed_responses = [
            {
                'StreamNames': ['stream-1', 'stream-2'],
                'StreamSummaries': [
                    {'StreamName': 'stream-1'},
                    {'StreamName': 'stream-2'},
                ],
                'HasMoreStreams': True,
                'NextToken': 'token',
            }
        ]
        expected_params = {'ExclusiveStartStreamName': 'stream-1'}
        stdout, _, _ = self.assert_params_for_cmd(cmdline, expected_params)
        output = json.dumps(stdout)
        self.assertIn('NextToken', output)
        self.assertIn('HasMoreStreams', output)

    def test_exclusive_start_stream_name_incompatible_with_page_args(self):
        cmdline = self.prefix + ['--exclusive-start-stream-name', 'stream-1']
        cmdline += ['--page-size', '1']
        _, stderr, _ = self.run_cmd(cmdline, expected_rc=252)
        self.assertIn('Cannot specify --no-paginate', stderr)
        self.assertIn('--page-size', stderr)


class TestListStreamsHelp(BaseAWSHelpOutputTest):
    def test_exclusive_start_stream_name_is_undocumented(self):
        self.driver.main(['kinesis', 'list-streams', 'help'])
        self.assert_not_contains('--exclusive-start-steam-name')



import os

from awscli import clidriver
from awscli.autocomplete import db, generator
from awscli.autocomplete.local.indexer import ModelIndexer
from awscli.autocomplete.serverside import model
from awscli.autocomplete.serverside.indexer import APICallIndexer
from awscli.testutils import unittest


def _ddb_only_command_table(command_table, **kwargs):
    for key in list(command_table):
        if key != 'dynamodb':
            del command_table[key]


class TestCanGenerateServerIndex(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.db_connection = db.DatabaseConnection(":memory:")
        index_generator = generator.IndexGenerator(
            [
                ModelIndexer(cls.db_connection),
                APICallIndexer(cls.db_connection),
            ],
        )
        driver = clidriver.create_clidriver()
        driver.session.register(
            'building-command-table.main', _ddb_only_command_table
        )
        index_generator.generate_index(driver)

    def test_can_query_model_from_index(self):
        lookup = model.DBCompletionLookup(self.db_connection)
        # We'll query a few lookups to ensure that we indexed them
        # correctly.
        result = lookup.get_server_completion_data(
            ['aws', 'dynamodb'], 'delete-table', 'table-name'
        )
        self.assertEqual(
            result,
            {
                'completions': [
                    {
                        'jp_expr': 'TableNames[]',
                        'operation': 'list_tables',
                        'parameters': {},
                        'service': 'dynamodb',
                    }
                ]
            },
        )
        result = lookup.get_server_completion_data(
            ['aws', 'dynamodb'], 'update-global-table', 'global-table-name'
        )
        self.assertEqual(
            result,
            {
                'completions': [
                    {
                        'jp_expr': 'GlobalTables[].GlobalTableName',
                        'operation': 'list_global_tables',
                        'parameters': {},
                        'service': 'dynamodb',
                    }
                ]
            },
        )

    def test_returns_none_if_no_lookup_data_found(self):
        lookup = model.DBCompletionLookup(self.db_connection)
        self.assertIsNone(
            lookup.get_server_completion_data(
                ['aws', 'dynamodb'], 'delete-table', 'unknown-param'
            )
        )
        self.assertIsNone(
            lookup.get_server_completion_data(
                ['aws', 'dynamodb'], 'unknown-operation', 'foo'
            )
        )
        self.assertIsNone(
            lookup.get_server_completion_data(
                ['aws', 'unknown-service'], 'unknown-operation', 'foo'
            )
        )


class TestCanHandleNoCompletionData(unittest.TestCase):
    def setUp(self):
        self.db_connection = db.DatabaseConnection(":memory:")

    def tearDown(self):
        self.db_connection.close()

    def _disable_cli_loaders(self, event_name, session, **kwargs):
        loader = session.get_component('data_loader')
        for path in loader.search_paths[::]:
            if path.endswith(os.path.join('awscli', 'botocore', 'data')):
                loader.search_paths.remove(path)

    def test_no_errors_when_missing_completion_data(self):
        index_generator = generator.IndexGenerator(
            [
                ModelIndexer(self.db_connection),
                APICallIndexer(self.db_connection),
            ],
        )
        driver = clidriver.create_clidriver()
        # We're going to remove the CLI data path from the loader.
        # This will result in the loader not being able to find any
        # completion data, which allows us to verify the behavior when
        # there's no completion data.
        driver.session.register(
            'building-command-table.main', _ddb_only_command_table
        )
        driver.session.register(
            'building-command-table.dynamodb', self._disable_cli_loaders
        )
        index_generator.generate_index(driver)
        # We shouldn't get any data now because we couldn't load
        # completion data.
        lookup = model.DBCompletionLookup(self.db_connection)
        result = lookup.get_server_completion_data(
            ['aws', 'dynamodb'], 'delete-table', 'table-name'
        )
        self.assertIsNone(result)



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
 # the License is located at
 #
 # http://aws.amazon.com/apache2.0/
 #
 # or in the "license" file accompanying this file. This file is
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 
from tests import BaseSessionTest
 
 
class TestBedrock(BaseSessionTest):
    def test_invoke_model_with_bidirectional_stream(self):
        """InvokeModelWithBidirectionalStream operation removed due to h2 requirement"""
        bedrock_runtime_client = self.session.create_client('bedrock-runtime', 'us-west-2')
        try:
            bedrock_runtime_client.invoke_model_with_bidirectional_stream()
        except AttributeError:
            pass
        else:
            self.fail(
                'invoke_model_with_bidirectional_stream shouldn\'t be available on the '
                'bedrock-runtime client.'
            )



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import argparse
import io
import json
import signal
from unittest import mock

from botocore.exceptions import ClientError, NoCredentialsError, NoRegionError
from ruamel.yaml import YAML

from awscli.arguments import UnknownArgumentError
from awscli.constants import (
    CLIENT_ERROR_RC,
    CONFIGURATION_ERROR_RC,
    PARAM_VALIDATION_ERROR_RC,
)
from awscli.customizations.exceptions import (
    ConfigurationError,
    ParamValidationError,
)
from awscli.errorhandler import (
    ClientErrorHandler,
    EnhancedErrorFormatter,
    construct_cli_error_handlers_chain,
)
from awscli.utils import PagerInitializationException
from tests.unit.test_clidriver import FakeSession


class TestClientErrorHandler:
    def setup_method(self):
        self.session = FakeSession()
        self.handler = ClientErrorHandler(self.session)

    def test_displays_structured_error_with_additional_members(self):
        error_response = {
            'Error': {
                'Code': 'NoSuchBucket',
                'Message': 'Error',
                'BucketName': 'my-bucket',
            },
            'ResponseMetadata': {'RequestId': '123'},
        }
        client_error = ClientError(error_response, 'GetObject')
        client_error.modeled_fields = {'Code', 'Message', 'BucketName'}

        stdout = io.StringIO()
        stderr = io.StringIO()

        rc = self.handler.handle_exception(client_error, stdout, stderr)

        assert rc == CLIENT_ERROR_RC
        expected = (
            '\n'
            'aws: [ERROR]: An error occurred (NoSuchBucket) '
            'when calling the GetObject operation: Error\n'
            '\n'
            'Additional error details:\n'
            'BucketName: my-bucket\n'
        )
        assert stderr.getvalue() == expected

    def test_displays_standard_error_without_additional_members(self):
        error_response = {
            'Error': {
                'Code': 'AccessDenied',
                'Message': 'Access Denied',
            },
            'ResponseMetadata': {'RequestId': '123'},
        }
        client_error = ClientError(error_response, 'GetObject')

        stdout = io.StringIO()
        stderr = io.StringIO()

        rc = self.handler.handle_exception(client_error, stdout, stderr)

        assert rc == CLIENT_ERROR_RC
        expected = (
            '\n'
            'aws: [ERROR]: An error occurred (AccessDenied) '
            'when calling the GetObject operation: Access Denied\n'
        )
        assert stderr.getvalue() == expected

    def test_respects_legacy_format_config(self):
        error_response = {
            'Error': {
                'Code': 'NoSuchBucket',
                'Message': 'Error',
                'BucketName': 'test',
            },
            'ResponseMetadata': {'RequestId': '123'},
        }
        client_error = ClientError(error_response, 'GetObject')

        self.session.session_vars['cli_error_format'] = 'legacy'

        stdout = io.StringIO()
        stderr = io.StringIO()

        rc = self.handler.handle_exception(client_error, stdout, stderr)

        assert rc == CLIENT_ERROR_RC
        expected = (
            '\n'
            'aws: [ERROR]: An error occurred (NoSuchBucket) '
            'when calling the GetObject operation: Error\n'
        )
        assert stderr.getvalue() == expected

    def test_error_format_case_insensitive(self):
        error_response = {
            'Error': {
                'Code': 'NoSuchBucket',
                'Message': 'Error',
                'BucketName': 'test',
            },
            'ResponseMetadata': {'RequestId': '123'},
        }
        client_error = ClientError(error_response, 'GetObject')
        client_error.modeled_fields = {'Code', 'Message', 'BucketName'}

        self.session.config_store.set_config_provider(
            'cli_error_format', mock.Mock(provide=lambda: 'Enhanced')
        )

        stdout = io.StringIO()
        stderr = io.StringIO()

        rc = self.handler.handle_exception(client_error, stdout, stderr)

        assert rc == CLIENT_ERROR_RC
        expected = (
            '\n'
            'aws: [ERROR]: An error occurred (NoSuchBucket) '
            'when calling the GetObject operation: Error\n'
            '\n'
            'Additional error details:\n'
            'BucketName: test\n'
        )
        assert stderr.getvalue() == expected

    def test_modeled_fields_filters_unmodeled_from_display(self):
        error_response = {
            'Error': {
                'Code': 'ExpiredToken',
                'Message': 'Token expired',
                'Token-0': 'AQoDYXdzEJr...sensitive...',
            },
            'ResponseMetadata': {'RequestId': '123'},
        }
        client_error = ClientError(error_response, 'ListBuckets')
        client_error.modeled_fields = {'Code', 'Message'}

        stdout = io.StringIO()
        stderr = io.StringIO()

        rc = self.handler.handle_exception(client_error, stdout, stderr)

        assert rc == CLIENT_ERROR_RC
        assert 'Token-0' not in stderr.getvalue()
        assert 'sensitive' not in stderr.getvalue()

    def test_modeled_fields_not_leaked_in_json_format(self):
        error_response = {
            'Error': {
                'Code': 'ExpiredToken',
                'Message': 'Token expired',
                'Token-0': 'AQoDYXdzEJr...sensitive...',
            },
            'ResponseMetadata': {'RequestId': '123'},
        }
        client_error = ClientError(error_response, 'ListBuckets')
        client_error.modeled_fields = {'Code', 'Message'}

        self.session.session_vars['cli_error_format'] = 'json'

        stdout = io.StringIO()
        stderr = io.StringIO()

        self.handler.handle_exception(client_error, stdout, stderr)

        parsed = json.loads(stderr.getvalue())
        assert 'Token-0' not in parsed
        assert '_modeled_fields' not in parsed
        assert 'modeled_fields' not in parsed

    def test_no_modeled_fields_hides_additional_fields(self):
        # ClientError without modeled_fields attribute (e.g. manually
        # constructed in customizations) should not show additional fields.
        error_response = {
            'Error': {
                'Code': 'CustomError',
                'Message': 'Something broke',
                'Detail': 'extra info',
            },
            'ResponseMetadata': {'RequestId': '123'},
        }
        client_error = ClientError(error_response, 'DoThing')

        stdout = io.StringIO()
        stderr = io.StringIO()

        self.handler.handle_exception(client_error, stdout, stderr)

        assert 'Detail' not in stderr.getvalue()

    def test_modeled_fields_case_insensitive_match(self):
        # Model has 'ErrorCode' but response has 'errorCode' (or vice versa)
        error_response = {
            'Error': {
                'Code': 'SomeError',
                'Message': 'msg',
                'errorcode': 'detail',
            },
            'ResponseMetadata': {'RequestId': '123'},
        }
        client_error = ClientError(error_response, 'Op')
        client_error.modeled_fields = {'Code', 'Message', 'ErrorCode'}

        stdout = io.StringIO()
        stderr = io.StringIO()

        self.handler.handle_exception(client_error, stdout, stderr)

        assert 'errorcode: detail' in stderr.getvalue()


class TestEnhancedErrorFormatter:
    def setup_method(self):
        self.formatter = EnhancedErrorFormatter()

    def test_format_error_with_no_additional_fields(self):
        error_info = {
            'Code': 'AccessDenied',
            'Message': 'Access Denied',
        }

        stream = io.StringIO()
        self.formatter.format_error(error_info, stream)

        output = stream.getvalue()
        assert output == ''

    def test_format_error_with_simple_fields(self):
        error_info = {
            'Code': 'NoSuchBucket',
            'Message': 'The bucket does not exist',
            'BucketName': 'my-bucket',
            'Region': 'us-east-1',
        }

        stream = io.StringIO()
        self.formatter.format_error(error_info, stream)

        output = stream.getvalue()
        expected = (
            '\n'
            'Additional error details:\n'
            'BucketName: my-bucket\n'
            'Region: us-east-1\n'
        )
        assert output == expected

    def test_format_error_with_small_list(self):
        error_info = {
            'Code': 'ValidationError',
            'Message': 'Validation failed',
            'AllowedValues': ['value1', 'value2', 'value3'],
        }

        stream = io.StringIO()
        self.formatter.format_error(error_info, stream)

        output = stream.getvalue()
        expected = (
            '\n'
            'Additional error details:\n'
            'AllowedValues: [value1, value2, value3]\n'
        )
        assert output == expected

    def test_format_error_with_small_dict(self):
        error_info = {
            'Code': 'ValidationError',
            'Message': 'Validation failed',
            'Metadata': {'key1': 'value1', 'key2': 'value2'},
        }

        stream = io.StringIO()
        self.formatter.format_error(error_info, stream)

        output = stream.getvalue()
        expected = (
            '\n'
            'Additional error details:\n'
            'Metadata: {key1: value1, key2: value2}\n'
        )
        assert output == expected

    def test_format_error_with_complex_object(self):
        error_info = {
            'Code': 'ValidationError',
            'Message': 'Validation failed',
            'Details': [1, 2, 3, 4, 5, 6],
        }

        stream = io.StringIO()
        self.formatter.format_error(error_info, stream)

        output = stream.getvalue()
        expected = (
            '\n'
            'Additional error details:\n'
            'Details: <complex value>\n'
            'Use "--cli-error-format json" or another error format '
            'to see the full details.\n'
        )
        assert output == expected

    def test_format_error_with_nested_dict(self):
        error_info = {
            'Code': 'ValidationError',
            'Message': 'Validation failed',
            'FieldErrors': {
                'email': {'pattern': 'invalid', 'required': True},
                'age': {'min': 0, 'max': 120},
            },
        }

        stream = io.StringIO()
        self.formatter.format_error(error_info, stream)

        output = stream.getvalue()
        expected = (
            '\n'
            'Additional error details:\n'
            'FieldErrors: <complex value>\n'
            'Use "--cli-error-format json" or another error format '
            'to see the full details.\n'
        )
        assert output == expected

    def test_format_error_with_list_of_dicts(self):
        error_info = {
            'Code': 'TransactionCanceledException',
            'Message': 'Transaction cancelled',
            'CancellationReasons': [
                {
                    'Code': 'ConditionalCheckFailed',
                    'Message': 'Check failed',
                },
                {
                    'Code': 'ItemCollectionSizeLimitExceeded',
                    'Message': 'Too large',
                },
            ],
        }

        stream = io.StringIO()
        self.formatter.format_error(error_info, stream)

        output = stream.getvalue()
        expected = (
            '\n'
            'Additional error details:\n'
            'CancellationReasons: <complex value>\n'
            'Use "--cli-error-format json" or another error format '
            'to see the full details.\n'
        )
        assert output == expected

    def test_format_error_with_mixed_types(self):
        error_info = {
            'Code': 'ComplexError',
            'Message': 'Complex error occurred',
            'StringField': 'test-value',
            'IntField': 42,
            'FloatField': 3.14,
            'BoolField': True,
            'NoneField': None,
        }

        stream = io.StringIO()
        self.formatter.format_error(error_info, stream)

        output = stream.getvalue()
        expected = (
            '\n'
            'Additional error details:\n'
            'StringField: test-value\n'
            'IntField: 42\n'
            'FloatField: 3.14\n'
            'BoolField: True\n'
            'NoneField: None\n'
        )
        assert output == expected

    def test_format_error_with_unicode_and_special_chars(self):
        error_info = {
            'Code': 'InvalidInput',
            'Message': 'Invalid input provided',
            'UserName': 'éîa',
            'Description': 'Error with "quotes" and \'apostrophes\'',
            'Path': '/path/to/file.txt',
        }

        stream = io.StringIO()
        self.formatter.format_error(error_info, stream)

        output = stream.getvalue()
        expected = (
            '\n'
            'Additional error details:\n'
            'UserName: éîa\n'
            'Description: Error with "quotes" and \'apostrophes\'\n'
            'Path: /path/to/file.txt\n'
        )
        assert output == expected

    def test_format_error_hides_unmodeled_fields(self):
        # Unmodeled fields are now filtered before reaching the formatter.
        # Formatter receives only modeled fields.
        error_info = {
            'Code': 'ExpiredToken',
            'Message': 'Token expired',
        }

        stream = io.StringIO()
        self.formatter.format_error(error_info, stream)

        assert stream.getvalue() == ''

    def test_format_error_shows_modeled_fields(self):
        # Unmodeled fields are filtered before reaching the formatter.
        error_info = {
            'Code': 'FileSystemNotFound',
            'Message': 'Not found',
            'ErrorCode': 'FileSystemNotFound',
        }

        stream = io.StringIO()
        self.formatter.format_error(error_info, stream)

        output = stream.getvalue()
        assert 'ErrorCode: FileSystemNotFound' in output

    def test_format_error_with_large_list(self):
        error_info = {
            'Code': 'LargeList',
            'Message': 'Large list error',
            'Items': list(range(10)),
        }

        stream = io.StringIO()
        self.formatter.format_error(error_info, stream)

        output = stream.getvalue()
        expected = (
            '\n'
            'Additional error details:\n'
            'Items: <complex value>\n'
            'Use "--cli-error-format json" or another error format '
            'to see the full details.\n'
        )
        assert output == expected


class TestRealWorldErrorScenarios:
    def setup_method(self):
        self.session = FakeSession()
        self.handler = ClientErrorHandler(self.session)

    def test_dynamodb_transaction_cancelled_error(self):
        error_response = {
            'Error': {
                'Code': 'TransactionCanceledException',
                'Message': (
                    'Transaction cancelled, please refer to '
                    'CancellationReasons for specific reasons'
                ),
            },
            'CancellationReasons': [
                {
                    'Code': 'ConditionalCheckFailed',
                    'Message': 'The conditional request failed',
                    'Item': {
                        'id': {'S': 'item-123'},
                        'status': {'S': 'active'},
                    },
                },
                {
                    'Code': 'None',
                    'Message': None,
                },
            ],
            'ResponseMetadata': {
                'RequestId': 'abc-123',
                'HTTPStatusCode': 400,
            },
        }
        client_error = ClientError(error_response, 'TransactWriteItems')
        client_error.modeled_fields = {
            'Code',
            'Message',
            'CancellationReasons',
        }

        stdout = io.StringIO()
        stderr = io.StringIO()

        rc = self.handler.handle_exception(client_error, stdout, stderr)

        assert rc == CLIENT_ERROR_RC
        expected = (
            '\n'
            'aws: [ERROR]: An error occurred (TransactionCanceledException) '
            'when calling the TransactWriteItems operation: '
            'Transaction cancelled, please refer to '
            'CancellationReasons for specific reasons\n'
            '\n'
            'Additional error details:\n'
            'CancellationReasons: <complex value>\n'
            'Use "--cli-error-format json" or another error format '
            'to see the full details.\n'
        )
        assert stderr.getvalue() == expected


class TestParsedGlobalsPassthrough:
    def test_error_handler_receives_parsed_globals_from_clidriver(self):
        session = FakeSession()

        parsed_globals = argparse.Namespace()
        parsed_globals.cli_error_format = 'json'
        parsed_globals.command = 's3'
        parsed_globals.color = 'auto'
        parsed_globals.query = None

        error_handler = construct_cli_error_handlers_chain(session)

        error_response = {
            'Error': {
                'Code': 'NoSuchBucket',
                'Message': 'The specified bucket does not exist',
                'BucketName': 'test-bucket',
            },
            'ResponseMetadata': {'RequestId': '123'},
        }
        client_error = ClientError(error_response, 'GetObject')
        client_error.modeled_fields = {'Code', 'Message', 'BucketName'}

        stdout = io.StringIO()
        stderr = io.StringIO()

        rc = error_handler.handle_exception(
            client_error, stdout, stderr, parsed_globals=parsed_globals
        )

        assert rc == CLIENT_ERROR_RC

        stderr_output = stderr.getvalue()
        parsed_json = json.loads(stderr_output)
        assert parsed_json['Code'] == 'NoSuchBucket'
        assert parsed_json['Message'] == 'The specified bucket does not exist'
        assert parsed_json['BucketName'] == 'test-bucket'

    def test_error_handler_without_parsed_globals_uses_default(self):
        session = FakeSession()

        error_handler = construct_cli_error_handlers_chain(session)

        error_response = {
            'Error': {
                'Code': 'NoSuchBucket',
                'Message': 'The specified bucket does not exist',
                'BucketName': 'test-bucket',
            },
            'ResponseMetadata': {'RequestId': '123'},
        }
        client_error = ClientError(error_response, 'GetObject')
        client_error.modeled_fields = {'Code', 'Message', 'BucketName'}

        stdout = io.StringIO()
        stderr = io.StringIO()

        rc = error_handler.handle_exception(
            client_error, stdout, stderr, parsed_globals=None
        )

        assert rc == CLIENT_ERROR_RC
        expected = (
            '\n'
            'aws: [ERROR]: An error occurred (NoSuchBucket) '
            'when calling the GetObject operation: '
            'The specified bucket does not exist\n'
            '\n'
            'Additional error details:\n'
            'BucketName: test-bucket\n'
        )
        assert stderr.getvalue() == expected


class TestNonModeledErrorStructuredFormatting:
    def setup_method(self):
        self.yaml = YAML(typ="safe", pure=True)

    def _load_yaml(self, content):
        return self.yaml.load(io.StringIO(content))

    def test_no_region_error_with_json_format(self):
        session = FakeSession()
        error_handler = construct_cli_error_handlers_chain(session)
        exception = NoRegionError()

        stdout = io.StringIO()
        stderr = io.StringIO()
        parsed_globals = argparse.Namespace()
        parsed_globals.cli_error_format = 'json'
        parsed_globals.query = None
        parsed_globals.color = 'auto'

        rc = error_handler.handle_exception(
            exception, stdout, stderr, parsed_globals=parsed_globals
        )

        assert rc == CONFIGURATION_ERROR_RC
        stderr_output = stderr.getvalue()
        parsed_json = json.loads(stderr_output)
        assert parsed_json['Code'] == 'NoRegion'
        assert 'aws configure' in parsed_json['Message']

    def test_no_credentials_error_with_yaml_format(self):
        session = FakeSession()
        error_handler = construct_cli_error_handlers_chain(session)
        exception = NoCredentialsError()

        stdout = io.StringIO()
        stderr = io.StringIO()
        parsed_globals = argparse.Namespace()
        parsed_globals.cli_error_format = 'yaml'
        parsed_globals.query = None
        parsed_globals.color = 'auto'

        rc = error_handler.handle_exception(
            exception, stdout, stderr, parsed_globals=parsed_globals
        )

        assert rc == CONFIGURATION_ERROR_RC
        stderr_output = stderr.getvalue()
        parsed_yaml = self._load_yaml(stderr_output)
        assert parsed_yaml['Code'] == 'NoCredentials'
        assert (
            'aws' in parsed_yaml['Message']
            and 'login' in parsed_yaml['Message']
        )

    def test_configuration_error_with_enhanced_format(self):
        session = FakeSession()

        error_handler = construct_cli_error_handlers_chain(session)
        exception = ConfigurationError('Invalid configuration value')

        stdout = io.StringIO()
        stderr = io.StringIO()
        parsed_globals = argparse.Namespace()
        parsed_globals.cli_error_format = 'enhanced'
        parsed_globals.query = None
        parsed_globals.color = 'auto'

        rc = error_handler.handle_exception(
            exception, stdout, stderr, parsed_globals=parsed_globals
        )

        assert rc == CONFIGURATION_ERROR_RC
        expected = (
            '\n'
            'aws: [ERROR]: An error occurred (Configuration): '
            'Invalid configuration value\n'
        )
        assert stderr.getvalue() == expected

    def test_pager_error_with_json_format(self):
        session = FakeSession()

        error_handler = construct_cli_error_handlers_chain(session)
        exception = PagerInitializationException('Pager not found')

        stdout = io.StringIO()
        stderr = io.StringIO()
        parsed_globals = argparse.Namespace()
        parsed_globals.cli_error_format = 'json'
        parsed_globals.query = None
        parsed_globals.color = 'auto'

        rc = error_handler.handle_exception(
            exception, stdout, stderr, parsed_globals=parsed_globals
        )

        assert rc == CONFIGURATION_ERROR_RC
        stderr_output = stderr.getvalue()
        parsed_json = json.loads(stderr_output)
        assert parsed_json['Code'] == 'Pager'
        assert 'Unable to redirect output to pager' in parsed_json['Message']

    def test_param_validation_error_with_yaml_format(self):
        session = FakeSession()

        error_handler = construct_cli_error_handlers_chain(session)
        exception = ParamValidationError('Invalid parameter value')

        stdout = io.StringIO()
        stderr = io.StringIO()
        parsed_globals = argparse.Namespace()
        parsed_globals.cli_error_format = 'yaml'
        parsed_globals.query = None
        parsed_globals.color = 'auto'

        rc = error_handler.handle_exception(
            exception, stdout, stderr, parsed_globals=parsed_globals
        )

        assert rc == PARAM_VALIDATION_ERROR_RC
        stderr_output = stderr.getvalue()
        parsed_yaml = self._load_yaml(stderr_output)
        assert parsed_yaml['Code'] == 'ParamValidation'
        assert 'Invalid parameter value' in parsed_yaml['Message']

    def test_error_codes_without_error_suffix(self):
        session = FakeSession()
        error_handler = construct_cli_error_handlers_chain(session)

        parsed_globals = argparse.Namespace()
        parsed_globals.cli_error_format = 'json'
        parsed_globals.query = None
        parsed_globals.color = 'auto'

        test_cases = [
            (NoRegionError(), 'NoRegion'),
            (NoCredentialsError(), 'NoCredentials'),
            (ConfigurationError('test'), 'Configuration'),
            (PagerInitializationException('test'), 'Pager'),
            (ParamValidationError('test'), 'ParamValidation'),
        ]

        for exception, expected_code in test_cases:
            stdout = io.StringIO()
            stderr = io.StringIO()

            error_handler.handle_exception(
                exception, stdout, stderr, parsed_globals=parsed_globals
            )

            stderr_output = stderr.getvalue()
            parsed_json = json.loads(stderr_output)
            assert parsed_json['Code'] == expected_code

    def test_unknown_argument_error_remains_plain_text(self):
        session = FakeSession()
        error_handler = construct_cli_error_handlers_chain(session)
        exception = UnknownArgumentError('--invalid-arg')

        stdout = io.StringIO()
        stderr = io.StringIO()
        parsed_globals = argparse.Namespace()
        parsed_globals.cli_error_format = 'json'
        parsed_globals.query = None
        parsed_globals.color = 'auto'

        rc = error_handler.handle_exception(
            exception, stdout, stderr, parsed_globals=parsed_globals
        )

        assert rc == PARAM_VALIDATION_ERROR_RC
        expected = (
            '\n'
            'usage: aws [options] <command> <subcommand> '
            '[<subcommand> ...] [parameters]\n'
            'To see help text, you can run:\n'
            '\n'
            '  aws help\n'
            '  aws <command> help\n'
            '  aws <command> <subcommand> help\n'
            '\n'
            '\n'
            'aws: [ERROR]: --invalid-arg\n'
        )
        assert stderr.getvalue() == expected

    def test_legacy_format_uses_plain_text(self):
        session = FakeSession()
        error_handler = construct_cli_error_handlers_chain(session)
        exception = NoRegionError()

        stdout = io.StringIO()
        stderr = io.StringIO()
        parsed_globals = argparse.Namespace()
        parsed_globals.cli_error_format = 'legacy'
        parsed_globals.query = None
        parsed_globals.color = 'auto'

        rc = error_handler.handle_exception(
            exception, stdout, stderr, parsed_globals=parsed_globals
        )

        assert rc == CONFIGURATION_ERROR_RC
        expected = (
            '\n'
            'aws: [ERROR]: You must specify a region. You can also '
            'configure your region by running "aws configure".\n'
        )
        assert stderr.getvalue() == expected

    def test_enhanced_format_includes_error_prefix(self):
        session = FakeSession()
        error_handler = construct_cli_error_handlers_chain(session)
        exception = NoRegionError()

        stdout = io.StringIO()
        stderr = io.StringIO()
        parsed_globals = argparse.Namespace()
        parsed_globals.cli_error_format = 'enhanced'
        parsed_globals.query = None
        parsed_globals.color = 'auto'

        rc = error_handler.handle_exception(
            exception, stdout, stderr, parsed_globals=parsed_globals
        )

        assert rc == CONFIGURATION_ERROR_RC
        expected = (
            '\n'
            'aws: [ERROR]: An error occurred (NoRegion): '
            'You must specify a region. You can also configure your region '
            'by running "aws configure".\n'
        )
        assert stderr.getvalue() == expected

    def test_interrupt_exception_remains_plain_text(self):
        session = FakeSession()
        error_handler = construct_cli_error_handlers_chain(session)
        exception = KeyboardInterrupt()

        stdout = io.StringIO()
        stderr = io.StringIO()
        parsed_globals = argparse.Namespace()
        parsed_globals.cli_error_format = 'json'
        parsed_globals.query = None
        parsed_globals.color = 'auto'

        rc = error_handler.handle_exception(
            exception, stdout, stderr, parsed_globals=parsed_globals
        )

        assert rc == 128 + signal.SIGINT
        assert stdout.getvalue() == "\n"
        assert stderr.getvalue() == ""



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/

import time

import pytest

from awscli.customizations.ecs.expressgateway.managedresource import (
    ManagedResource,
)
from awscli.customizations.ecs.expressgateway.managedresourcegroup import (
    ManagedResourceGroup,
)
from awscli.customizations.ecs.expressgateway.stream_display import (
    StreamDisplay,
)


@pytest.fixture
def display():
    """Fixture that returns a StreamDisplay with color enabled."""
    return StreamDisplay(use_color=True)


class TestStreamDisplay:
    """Test StreamDisplay for text-based monitoring output."""

    def test_show_startup_message(self, display, capsys):
        """Test startup message includes timestamp"""
        display.show_startup_message()

        output = capsys.readouterr().out
        assert "Starting monitoring..." in output
        assert "[" in output

    def test_show_polling_message(self, display, capsys):
        """Test polling message includes timestamp"""
        display.show_polling_message()

        output = capsys.readouterr().out
        assert "Polling for updates..." in output

    def test_show_monitoring_data_with_info(self, display, capsys):
        """Test showing info message"""
        display.show_monitoring_data(None, "Info message")

        output = capsys.readouterr().out
        assert "Info message" in output and output.endswith("\n")

    def test_show_monitoring_data_first_poll_shows_all(self, display, capsys):
        """Test first poll shows all resources"""
        resource = ManagedResource(
            "LoadBalancer", "lb-123", "ACTIVE", None, None
        )
        resource_group = ManagedResourceGroup(resources=[resource])

        display.show_monitoring_data(resource_group, None)

        output = capsys.readouterr().out
        assert "LoadBalancer" in output
        assert "lb-123" in output

    def test_show_monitoring_data_no_changes(self, display, capsys):
        """Test no output when resources haven't changed"""
        resource = ManagedResource(
            "LoadBalancer", "lb-123", "ACTIVE", None, None
        )
        resource_group = ManagedResourceGroup(resources=[resource])

        # First poll - show all
        display.show_monitoring_data(resource_group, None)
        capsys.readouterr()  # Clear output to test second call

        # Second poll - same resources, no changes
        display.show_monitoring_data(resource_group, None)

        output = capsys.readouterr().out
        assert output == ""

    def test_show_monitoring_data_with_new_resource(self, display, capsys):
        """Test output when new resources are added"""
        resource1 = ManagedResource(
            "LoadBalancer", "lb-123", "ACTIVE", None, None
        )
        resource_group1 = ManagedResourceGroup(resources=[resource1])

        display.show_monitoring_data(resource_group1, None)
        capsys.readouterr()  # Clear initial output to verify new resource shows

        # Second resource group with additional resource
        resource2 = ManagedResource(
            "TargetGroup", "tg-456", "ACTIVE", None, None
        )
        resource_group2 = ManagedResourceGroup(
            resources=[resource1, resource2]
        )

        display.show_monitoring_data(resource_group2, None)

        output = capsys.readouterr().out
        assert "TargetGroup" in output

    def test_show_timeout_message(self, display, capsys):
        """Test timeout message"""
        display.show_timeout_message()

        output = capsys.readouterr().out
        assert "timeout reached" in output.lower()

    def test_show_service_inactive_message(self, display, capsys):
        """Test service inactive message"""
        display.show_service_inactive_message()

        output = capsys.readouterr().out
        assert "inactive" in output.lower()

    def test_show_completion_message(self, display, capsys):
        """Test completion message"""
        display.show_completion_message()

        output = capsys.readouterr().out
        assert "complete" in output.lower()

    def test_show_user_stop_message(self, display, capsys):
        """Test user stop message"""
        display.show_user_stop_message()

        output = capsys.readouterr().out
        assert "stopped by user" in output.lower()

    def test_show_error_message(self, display, capsys):
        """Test error message"""
        display.show_error_message("Test error")

        output = capsys.readouterr().out
        assert "Error" in output
        assert "Test error" in output

    def test_use_color_parameter(self):
        """Test use_color parameter is stored"""
        display_with_color = StreamDisplay(use_color=True)
        assert display_with_color.use_color is True

        display_no_color = StreamDisplay(use_color=False)
        assert display_no_color.use_color is False

    def test_print_flattened_resources_with_reason(self, display, capsys):
        """Test resource with reason prints on separate line"""
        resource = ManagedResource(
            "LoadBalancer",
            "lb-123",
            "CREATING",
            None,
            "Waiting for DNS propagation",
        )
        resource_group = ManagedResourceGroup(resources=[resource])

        display.show_monitoring_data(resource_group, None)

        output = capsys.readouterr().out
        lines = output.splitlines()

        # Find the line with LoadBalancer
        lb_line_idx = next(
            i for i, line in enumerate(lines) if "LoadBalancer" in line
        )
        reason_line_idx = next(
            i
            for i, line in enumerate(lines)
            if "Reason: Waiting for DNS propagation" in line
        )

        # Reason should be on a different line than LoadBalancer
        assert lb_line_idx != reason_line_idx
        # Reason should come after the LoadBalancer line
        assert reason_line_idx > lb_line_idx

    def test_print_flattened_resources_with_updated_at(self, display, capsys):
        """Test resource with updated_at timestamp prints on separate line"""
        current_time = time.time()
        resource = ManagedResource(
            "LoadBalancer", "lb-123", "ACTIVE", current_time, None
        )
        resource_group = ManagedResourceGroup(resources=[resource])

        display.show_monitoring_data(resource_group, None)

        output = capsys.readouterr().out
        lines = output.splitlines()

        # Find the line with LoadBalancer
        lb_line_idx = next(
            i for i, line in enumerate(lines) if "LoadBalancer" in line
        )
        updated_line_idx = next(
            i for i, line in enumerate(lines) if "Last Updated At:" in line
        )

        # Updated timestamp should be on a different line than LoadBalancer
        assert lb_line_idx != updated_line_idx
        # Updated timestamp should come after the LoadBalancer line
        assert updated_line_idx > lb_line_idx

    def test_print_flattened_resources_with_additional_info(
        self, display, capsys
    ):
        """Test resource with additional_info prints on separate line"""
        resource = ManagedResource(
            "LoadBalancer", "lb-123", "ACTIVE", None, None
        )
        resource.additional_info = "DNS: example.elb.amazonaws.com"
        resource_group = ManagedResourceGroup(resources=[resource])

        display.show_monitoring_data(resource_group, None)

        output = capsys.readouterr().out
        lines = output.splitlines()

        # Find the line with LoadBalancer
        lb_line_idx = next(
            i for i, line in enumerate(lines) if "LoadBalancer" in line
        )
        info_line_idx = next(
            i
            for i, line in enumerate(lines)
            if "Info: DNS: example.elb.amazonaws.com" in line
        )

        # Info should be on a different line than LoadBalancer
        assert lb_line_idx != info_line_idx
        # Info should come after the LoadBalancer line
        assert info_line_idx > lb_line_idx

    def test_print_flattened_resources_complete_multi_line(
        self, display, capsys
    ):
        """Test resource with all fields prints on multiple lines"""
        resource = ManagedResource(
            "LoadBalancer",
            "lb-123",
            "CREATING",
            time.time(),
            "Provisioning load balancer",
        )
        resource.additional_info = "Type: application"
        resource_group = ManagedResourceGroup(resources=[resource])

        display.show_monitoring_data(resource_group, None)

        output = capsys.readouterr().out
        lines = output.splitlines()

        # Find all the relevant lines
        lb_line_idx = next(
            i
            for i, line in enumerate(lines)
            if "LoadBalancer" in line and "lb-123" in line
        )
        reason_line_idx = next(
            i
            for i, line in enumerate(lines)
            if "Reason: Provisioning load balancer" in line
        )
        updated_line_idx = next(
            i for i, line in enumerate(lines) if "Last Updated At:" in line
        )
        info_line_idx = next(
            i
            for i, line in enumerate(lines)
            if "Info: Type: application" in line
        )

        # All detail lines should be different from the main line
        assert reason_line_idx != lb_line_idx
        assert updated_line_idx != lb_line_idx
        assert info_line_idx != lb_line_idx

        # All detail lines should come after the main line
        assert reason_line_idx > lb_line_idx
        assert updated_line_idx > lb_line_idx
        assert info_line_idx > lb_line_idx

    def test_diff_detects_status_change(self, display, capsys):
        """Test diff detects when status changes"""
        resource1 = ManagedResource(
            "LoadBalancer", "lb-123", "CREATING", None, None
        )
        resource_group1 = ManagedResourceGroup(resources=[resource1])

        # First poll
        display.show_monitoring_data(resource_group1, None)
        capsys.readouterr()  # Clear initial output to test status change detection

        # Second poll - same resource but status changed
        resource2 = ManagedResource(
            "LoadBalancer", "lb-123", "ACTIVE", None, None
        )
        resource_group2 = ManagedResourceGroup(resources=[resource2])

        display.show_monitoring_data(resource_group2, None)

        output = capsys.readouterr().out
        assert "LoadBalancer" in output
        assert "ACTIVE" in output

    def test_diff_detects_reason_change(self, display, capsys):
        """Test diff detects when reason changes"""
        resource1 = ManagedResource(
            "LoadBalancer", "lb-123", "CREATING", None, "Creating resources"
        )
        resource_group1 = ManagedResourceGroup(resources=[resource1])

        # First poll
        display.show_monitoring_data(resource_group1, None)
        capsys.readouterr()  # Clear initial output to test reason change detection

        # Second poll - same resource but reason changed
        resource2 = ManagedResource(
            "LoadBalancer", "lb-123", "CREATING", None, "Waiting for DNS"
        )
        resource_group2 = ManagedResourceGroup(resources=[resource2])

        display.show_monitoring_data(resource_group2, None)

        output = capsys.readouterr().out
        assert "Waiting for DNS" in output

    def test_diff_detects_additional_info_change(self, display, capsys):
        """Test diff detects when additional_info changes"""
        resource1 = ManagedResource(
            "LoadBalancer", "lb-123", "ACTIVE", None, None
        )
        resource1.additional_info = "DNS: old.example.com"
        resource_group1 = ManagedResourceGroup(resources=[resource1])

        # First poll
        display.show_monitoring_data(resource_group1, None)
        capsys.readouterr()  # Clear initial output to test additional_info change detection

        # Second poll - same resource but additional_info changed
        resource2 = ManagedResource(
            "LoadBalancer", "lb-123", "ACTIVE", None, None
        )
        resource2.additional_info = "DNS: new.example.com"
        resource_group2 = ManagedResourceGroup(resources=[resource2])

        display.show_monitoring_data(resource_group2, None)

        output = capsys.readouterr().out
        assert "new.example.com" in output

    def test_resource_with_none_type_shows_identifier(self, display, capsys):
        """Test resources with resource_type=None show identifier without type"""
        resource = ManagedResource(
            None,
            "mystery-resource-123",
            "FAILED",
            reason="Something went wrong",
        )
        resource_group = ManagedResourceGroup(
            resource_type="TestGroup", resources=[resource]
        )

        display.show_monitoring_data(resource_group, None)

        output = capsys.readouterr().out
        assert "mystery-resource-123" in output
        assert "FAILED" in output

    def test_resource_with_none_type_and_none_identifier(
        self, display, capsys
    ):
        """Test resources with both resource_type=None and identifier=None show 'Unknown Resource' placeholder"""
        resource = ManagedResource(None, None, "ACTIVE")
        resource_group = ManagedResourceGroup(
            resource_type="TestGroup", resources=[resource]
        )

        display.show_monitoring_data(resource_group, None)

        output = capsys.readouterr().out
        assert "Unknown Resource" in output
        assert "ACTIVE" in output



#!/usr/bin/env python
# Copyright 2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestAddOptionGroup(BaseAWSCommandParamsTest):
    maxDiff = None
    # This tests the customization where modify-option-group
    # was split into two commands add-option-group and
    # remove-option-group.  This class is testing add-option-gruop

    prefix = 'rds add-option-to-option-group '

    def test_add_option(self):
        args = (
            '--option-group-name myoptiongroup2 '
            '--options {"OptionName":"TDE"}'
        )
        cmdline = self.prefix + args
        result = {
            'OptionsToInclude': [{'OptionName': 'TDE'}],
            'OptionGroupName': 'myoptiongroup2',
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_option_to_remove_is_not_allowed(self):
        args = '--option-group-name myoptiongroup2 ' '--options-to-remove foo'
        cmdline = self.prefix + args
        self.assert_params_for_cmd(
            cmdline,
            expected_rc=252,
            stderr_contains='Unknown options: --options-to-remove',
        )


class TestRemoveOptionGroup(BaseAWSCommandParamsTest):
    prefix = 'rds remove-option-from-option-group '

    def test_remove_options(self):
        args = '--option-group-name myoptiongroup2 ' '--options TDE'
        cmdline = self.prefix + args
        result = {
            'OptionsToRemove': ['TDE'],
            'OptionGroupName': 'myoptiongroup2',
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_option_to_add_is_not_allowed(self):
        args = (
            '--option-group-name myoptiongroup2 '
            '--options-to-include {"OptionName":"TDE"}'
        )
        cmdline = self.prefix + args
        self.assert_params_for_cmd(
            cmdline,
            expected_rc=252,
            stderr_contains='Unknown options: --options-to-include',
        )



#!/usr/bin/env python
# Copyright 2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestModifyInstanceAttribute(BaseAWSCommandParamsTest):
    prefix = 'ec2 modify-instance-attribute '

    def setUp(self):
        super(TestModifyInstanceAttribute, self).setUp()
        self.expected_result = {
            'InstanceId': 'i-1234',
            'InstanceInitiatedShutdownBehavior': {'Value': 'terminate'},
        }

    def test_json_version(self):
        cmdline = self.prefix
        cmdline += '--instance-id i-1234 '
        cmdline += (
            '--instance-initiated-shutdown-behavior {"Value":"terminate"}'
        )
        self.assert_params_for_cmd(cmdline, self.expected_result)

    def test_shorthand_version(self):
        cmdline = self.prefix
        cmdline += '--instance-id i-1234 '
        cmdline += '--instance-initiated-shutdown-behavior Value=terminate'
        self.assert_params_for_cmd(cmdline, self.expected_result)

    def test_value_not_needed(self):
        # For structs of a single param value, you can skip the keep name,
        # so instead of Value=terminate, you can just say terminate.
        cmdline = self.prefix
        cmdline += '--instance-id i-1234 '
        cmdline += '--instance-initiated-shutdown-behavior terminate'
        self.assert_params_for_cmd(cmdline, self.expected_result)

    def test_boolean_value_in_top_level_true(self):
        # Just like everything else in argparse, the last value provided
        # for a destination has precedence.
        cmdline = self.prefix
        cmdline += '--instance-id i-1234 '
        cmdline += '--ebs-optimized Value=true'
        result = {'InstanceId': 'i-1234', 'EbsOptimized': {'Value': True}}
        self.assert_params_for_cmd(cmdline, result)

    def test_boolean_value_is_top_level_false(self):
        cmdline = self.prefix
        cmdline += '--instance-id i-1234 '
        cmdline += '--ebs-optimized Value=false'
        result = {'InstanceId': 'i-1234', 'EbsOptimized': {'Value': False}}
        self.assert_params_for_cmd(cmdline, result)

    def test_boolean_value_in_top_level_true_json(self):
        # Just like everything else in argparse, the last value provided
        # for a destination has precedence.
        cmdline = self.prefix
        cmdline += '--instance-id i-1234 '
        cmdline += '--ebs-optimized {"Value":true}'
        result = {'InstanceId': 'i-1234', 'EbsOptimized': {'Value': True}}
        self.assert_params_for_cmd(cmdline, result)

    def test_boolean_value_is_top_level_false_json(self):
        cmdline = self.prefix
        cmdline += '--instance-id i-1234 '
        cmdline += '--ebs-optimized {"Value":false}'
        result = {'InstanceId': 'i-1234', 'EbsOptimized': {'Value': False}}
        self.assert_params_for_cmd(cmdline, result)

    def test_boolean_param_top_level_true_no_value(self):
        cmdline = self.prefix
        cmdline += '--instance-id i-1234 '
        cmdline += '--ebs-optimized'
        result = {'InstanceId': 'i-1234', 'EbsOptimized': {'Value': True}}
        self.assert_params_for_cmd(cmdline, result)

    def test_boolean_param_top_level_false_no_value(self):
        cmdline = self.prefix
        cmdline += '--instance-id i-1234 '
        cmdline += '--no-ebs-optimized'
        result = {'InstanceId': 'i-1234', 'EbsOptimized': {'Value': False}}
        self.assert_params_for_cmd(cmdline, result)

    def test_mix_value_non_value_boolean_param(self):
        cmdline = self.prefix
        cmdline += '--instance-id i-1234 '
        # Can't mix non-value + value version of the arg.
        cmdline += '--no-ebs-optimized '
        cmdline += '--ebs-optimized Value=true'
        self.assert_params_for_cmd(
            cmdline, expected_rc=252, stderr_contains='Cannot specify both'
        )

    def test_mix_non_value_bools_not_allowed(self):
        cmdline = self.prefix
        cmdline += '--instance-id i-1234 '
        # Can't mix non-value + value version of the arg.
        cmdline += '--no-ebs-optimized '
        cmdline += '--ebs-optimized '
        self.assert_params_for_cmd(
            cmdline, expected_rc=252, stderr_contains='Cannot specify both'
        )


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# Copyright 2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestDescribeStacks(BaseAWSCommandParamsTest):
    prefix = 'cloudformation describe-stacks '

    def test_can_single_argument(self):
        cmdline = self.prefix + '--stack-name test-stack'
        result = {'StackName': 'test-stack'}
        self.assert_params_for_cmd(cmdline, result)

    def test_can_specify_no_paginate(self):
        cmdline = self.prefix + '--stack-name test-stack --no-paginate'
        result = {'StackName': 'test-stack'}
        self.assert_params_for_cmd(cmdline, result)



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import glob
import os
import shutil
import sys
import tempfile
from argparse import Namespace

import botocore
from botocore.compat import OrderedDict

import awscli.customizations.eks.kubeconfig as kubeconfig
from awscli.customizations.eks.exceptions import EKSClusterError, EKSError
from awscli.customizations.eks.ordered_yaml import ordered_yaml_load
from awscli.customizations.eks.update_kubeconfig import (
    API_VERSION,
    EKSClient,
    KubeconfigSelector,
)
from awscli.customizations.utils import uni_print
from awscli.testutils import mock, unittest
from tests.functional.eks.test_util import (
    describe_cluster_creating_response,
    describe_cluster_deleting_response,
    describe_cluster_no_status_response,
    describe_cluster_response,
    describe_cluster_response_outpost_cluster,
    get_testdata,
)


def generate_env_variable(files):
    """
    Generate a string which is an environment variable
    containing the absolute paths for each file in files

    :param files: The names of the files to put in the environment variable
    :type files: list
    """
    output = ""
    for file in files:
        if len(output) == 0:
            output = file
        else:
            output += os.path.pathsep + file
    return output


EXAMPLE_ARN = "arn:aws:eks:region:111222333444:cluster/ExampleCluster"


class TestKubeconfigSelector(unittest.TestCase):
    def setUp(self):
        self._validator = kubeconfig.KubeconfigValidator()
        self._loader = kubeconfig.KubeconfigLoader(self._validator)

    def assert_chosen_path(
        self, env_variable, path_in, cluster_name, chosen_path
    ):
        selector = KubeconfigSelector(
            env_variable, path_in, self._validator, self._loader
        )
        self.assertEqual(
            selector.choose_kubeconfig(cluster_name).path, chosen_path
        )

    def test_parse_env_variable(self):
        paths = [
            "",
            "",
            get_testdata("valid_bad_cluster"),
            get_testdata("valid_bad_cluster2"),
            "",
            get_testdata("valid_existing"),
            "",
        ]

        env_variable = generate_env_variable(paths)

        selector = KubeconfigSelector(
            env_variable, None, self._validator, self._loader
        )
        self.assertEqual(
            selector._paths, [path for path in paths if len(path) > 0]
        )

    def test_choose_env_only(self):
        paths = (
            [get_testdata("valid_simple"), get_testdata("valid_existing")]
            + glob.glob(get_testdata("invalid_*"))
            + [
                get_testdata("valid_bad_context"),
                get_testdata("valid_no_user"),
            ]
        )
        env_variable = generate_env_variable(paths)
        self.assert_chosen_path(
            env_variable, None, EXAMPLE_ARN, get_testdata("valid_simple")
        )

    def test_choose_existing(self):
        paths = (
            [get_testdata("valid_simple"), get_testdata("valid_existing")]
            + glob.glob(get_testdata("invalid_*"))
            + [
                get_testdata("valid_bad_context"),
                get_testdata("valid_no_user"),
                get_testdata("output_single"),
                get_testdata("output_single_with_role"),
            ]
        )
        env_variable = generate_env_variable(paths)
        self.assert_chosen_path(
            env_variable, None, EXAMPLE_ARN, get_testdata("output_single")
        )

    def test_arg_override(self):
        paths = (
            [get_testdata("valid_simple"), get_testdata("valid_existing")]
            + glob.glob(get_testdata("invalid_*"))
            + [
                get_testdata("valid_bad_context"),
                get_testdata("valid_no_user"),
                get_testdata("output_single"),
                get_testdata("output_single_with_role"),
            ]
        )
        env_variable = generate_env_variable(paths)
        self.assert_chosen_path(
            env_variable,
            get_testdata("output_combined"),
            EXAMPLE_ARN,
            get_testdata("output_combined"),
        )

    def test_first_corrupted(self):
        paths = glob.glob(get_testdata("invalid_*")) + [
            get_testdata("valid_bad_context"),
            get_testdata("valid_no_user"),
        ]
        env_variable = generate_env_variable(paths)
        selector = KubeconfigSelector(
            env_variable, None, self._validator, self._loader
        )
        self.assertRaises(
            kubeconfig.KubeconfigCorruptedError,
            selector.choose_kubeconfig,
            EXAMPLE_ARN,
        )

    def test_arg_override_first_corrupted(self):
        paths = glob.glob(get_testdata("invalid_*")) + [
            get_testdata("valid_bad_context"),
            get_testdata("valid_no_user"),
        ]
        env_variable = generate_env_variable(paths)
        self.assert_chosen_path(
            env_variable,
            get_testdata("output_combined"),
            EXAMPLE_ARN,
            get_testdata("output_combined"),
        )


class TestEKSClient(unittest.TestCase):
    def setUp(self):
        self._correct_cluster_entry = OrderedDict(
            [
                (
                    "cluster",
                    OrderedDict(
                        [
                            (
                                "certificate-authority-data",
                                describe_cluster_response()["cluster"][
                                    "certificateAuthority"
                                ]["data"],
                            ),
                            (
                                "server",
                                describe_cluster_response()["cluster"][
                                    "endpoint"
                                ],
                            ),
                        ]
                    ),
                ),
                ("name", describe_cluster_response()["cluster"]["arn"]),
            ]
        )
        self._correct_user_entry = OrderedDict(
            [
                ("name", describe_cluster_response()["cluster"]["arn"]),
                (
                    "user",
                    OrderedDict(
                        [
                            (
                                "exec",
                                OrderedDict(
                                    [
                                        ("apiVersion", API_VERSION),
                                        (
                                            "args",
                                            [
                                                "--region",
                                                "region",
                                                "eks",
                                                "get-token",
                                                "--cluster-name",
                                                "ExampleCluster",
                                                "--output",
                                                "json",
                                            ],
                                        ),
                                        ("command", "aws"),
                                    ]
                                ),
                            )
                        ]
                    ),
                ),
            ]
        )

        self._correct_user_entry_outpost_cluster = OrderedDict(
            [
                ("name", describe_cluster_response()["cluster"]["arn"]),
                (
                    "user",
                    OrderedDict(
                        [
                            (
                                "exec",
                                OrderedDict(
                                    [
                                        ("apiVersion", API_VERSION),
                                        (
                                            "args",
                                            [
                                                "--region",
                                                "region",
                                                "eks",
                                                "get-token",
                                                "--cluster-id",
                                                "aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee",
                                                "--output",
                                                "json",
                                            ],
                                        ),
                                        ("command", "aws"),
                                    ]
                                ),
                            )
                        ]
                    ),
                ),
            ]
        )

        self._mock_client = mock.Mock()
        self._mock_client.describe_cluster.return_value = (
            describe_cluster_response()
        )

        self._session = mock.Mock(spec=botocore.session.Session)
        self._session.create_client.return_value = self._mock_client
        self._session.profile = None

        self._client = EKSClient(
            self._session,
            parsed_args=Namespace(
                cluster_name="ExampleCluster", role_arn=None, proxy_url=None
            ),
        )

    def test_get_cluster_description(self):
        self.assertEqual(
            self._client.cluster_description,
            describe_cluster_response()["cluster"],
        )
        self._mock_client.describe_cluster.assert_called_once_with(
            name="ExampleCluster"
        )
        self._session.create_client.assert_called_once_with("eks")

    def test_get_cluster_description_no_status(self):
        self._mock_client.describe_cluster.return_value = (
            describe_cluster_no_status_response()
        )
        with self.assertRaises(EKSClusterError):
            self._client.cluster_description
        self._mock_client.describe_cluster.assert_called_once_with(
            name="ExampleCluster"
        )
        self._session.create_client.assert_called_once_with("eks")

    def test_get_cluster_entry(self):
        self.assertEqual(
            self._client.get_cluster_entry(), self._correct_cluster_entry
        )
        self._mock_client.describe_cluster.assert_called_once_with(
            name="ExampleCluster"
        )
        self._session.create_client.assert_called_once_with("eks")

    def test_get_cluster_entry_with_proxy_url_passed(self):
        proxy_url = "https://myproxy.com"
        correct_cluster_entry_with_proxy_url = OrderedDict(
            [
                (
                    "cluster",
                    OrderedDict(
                        [
                            (
                                "certificate-authority-data",
                                describe_cluster_response()["cluster"][
                                    "certificateAuthority"
                                ]["data"],
                            ),
                            (
                                "server",
                                describe_cluster_response()["cluster"][
                                    "endpoint"
                                ],
                            ),
                            ("proxy-url", proxy_url),
                        ]
                    ),
                ),
                ("name", describe_cluster_response()["cluster"]["arn"]),
            ]
        )
        client = EKSClient(
            self._session,
            parsed_args=Namespace(
                cluster_name="ProxiedCluster",
                role_arn=None,
                proxy_url=proxy_url,
            ),
        )
        self.assertEqual(
            client.get_cluster_entry(), correct_cluster_entry_with_proxy_url
        )
        self._mock_client.describe_cluster.assert_called_once_with(
            name="ProxiedCluster"
        )
        self._session.create_client.assert_called_once_with("eks")

    def test_get_user_entry(self):
        self.assertEqual(
            self._client.get_user_entry(), self._correct_user_entry
        )
        self._mock_client.describe_cluster.assert_called_once_with(
            name="ExampleCluster"
        )
        self._session.create_client.assert_called_once_with("eks")

    def test_get_user_entry_outpost_cluster(self):
        self._mock_client.describe_cluster.return_value = (
            describe_cluster_response_outpost_cluster()
        )
        self.assertEqual(
            self._client.get_user_entry(),
            self._correct_user_entry_outpost_cluster,
        )
        self._mock_client.describe_cluster.assert_called_once_with(
            name="ExampleCluster"
        )
        self._session.create_client.assert_called_once_with("eks")

    def test_get_both(self):
        self.assertEqual(
            self._client.get_cluster_entry(), self._correct_cluster_entry
        )
        self.assertEqual(
            self._client.get_user_entry(), self._correct_user_entry
        )
        self._mock_client.describe_cluster.assert_called_once_with(
            name="ExampleCluster"
        )
        self._session.create_client.assert_called_once_with("eks")

    def test_cluster_creating(self):
        self._mock_client.describe_cluster.return_value = (
            describe_cluster_creating_response()
        )
        with self.assertRaises(EKSClusterError):
            self._client.cluster_description
        self._mock_client.describe_cluster.assert_called_once_with(
            name="ExampleCluster"
        )
        self._session.create_client.assert_called_once_with("eks")

    def test_cluster_deleting(self):
        self._mock_client.describe_cluster.return_value = (
            describe_cluster_deleting_response()
        )
        with self.assertRaises(EKSClusterError):
            self._client.cluster_description
        self._mock_client.describe_cluster.assert_called_once_with(
            name="ExampleCluster"
        )
        self._session.create_client.assert_called_once_with("eks")

    def test_profile(self):
        self._session.profile = "profile"
        self._correct_user_entry["user"]["exec"]["env"] = [
            OrderedDict([("name", "AWS_PROFILE"), ("value", "profile")])
        ]
        self.assertEqual(
            self._client.get_user_entry(), self._correct_user_entry
        )
        self._mock_client.describe_cluster.assert_called_once_with(
            name="ExampleCluster"
        )
        self._session.create_client.assert_called_once_with("eks")

    def test_create_user_with_alias(self):
        self._correct_user_entry["name"] = "alias"
        self.assertEqual(
            self._client.get_user_entry(user_alias="alias"),
            self._correct_user_entry,
        )
        self._mock_client.describe_cluster.assert_called_once_with(
            name="ExampleCluster"
        )
        self._session.create_client.assert_called_once_with("eks")

    def test_create_user_without_alias(self):
        self.assertEqual(
            self._client.get_user_entry(), self._correct_user_entry
        )
        self._mock_client.describe_cluster.assert_called_once_with(
            name="ExampleCluster"
        )
        self._session.create_client.assert_called_once_with("eks")



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0
import base64

import pytest
from awscrt import checksums as crt_checksums
from s3transfer.checksums import (
    FullObjectChecksum,
    FullObjectChecksumCombiner,
    create_checksum_for_algorithm,
    resolve_full_object_checksum,
)
from s3transfer.exceptions import S3DownloadChecksumError


def _compute_expected_b64(data, algorithm):
    crc_fns = {
        'crc32': (crt_checksums.crc32, 4),
        'crc32c': (crt_checksums.crc32c, 4),
        'crc64nvme': (crt_checksums.crc64nvme, 8),
    }
    fn, byte_length = crc_fns[algorithm]
    crc = fn(data)
    return base64.b64encode(crc.to_bytes(byte_length, byteorder='big')).decode(
        'ascii'
    )


def _register_parts(combiner, parts, algorithm):
    for i, part_data in enumerate(parts):
        checksum = create_checksum_for_algorithm(algorithm)
        checksum.update(part_data)
        combiner.register_part(i, checksum, len(part_data))


class TestResolveFullObjectChecksum:
    def test_full_object_crc32(self):
        response = {
            'ChecksumType': 'FULL_OBJECT',
            'ChecksumCRC32': 'abc123==',
        }
        result = resolve_full_object_checksum(response)
        assert result == FullObjectChecksum(
            algorithm='crc32', expected_b64='abc123=='
        )

    def test_full_object_crc32c(self):
        response = {
            'ChecksumType': 'FULL_OBJECT',
            'ChecksumCRC32C': 'xyz789==',
        }
        result = resolve_full_object_checksum(response)
        assert result == FullObjectChecksum(
            algorithm='crc32c', expected_b64='xyz789=='
        )

    def test_full_object_crc64nvme(self):
        response = {
            'ChecksumType': 'FULL_OBJECT',
            'ChecksumCRC64NVME': 'nvme64==',
        }
        result = resolve_full_object_checksum(response)
        assert result == FullObjectChecksum(
            algorithm='crc64nvme', expected_b64='nvme64=='
        )

    def test_missing_checksum_type(self):
        assert resolve_full_object_checksum({}) is None

    def test_composite_checksum(self):
        response = {
            'ChecksumType': 'COMPOSITE',
            'ChecksumCRC32': 'abc123==',
        }
        assert resolve_full_object_checksum(response) is None

    def test_full_object_sha_only(self):
        response = {
            'ChecksumType': 'FULL_OBJECT',
            'ChecksumSHA256': 'sha256value==',
        }
        assert resolve_full_object_checksum(response) is None

    def test_case_insensitive_checksum_type(self):
        response = {
            'ChecksumType': 'full_object',
            'ChecksumCRC32': 'abc123==',
        }
        result = resolve_full_object_checksum(response)
        assert result is not None
        assert result.algorithm == 'crc32'


class TestCreateChecksumForAlgorithm:
    @pytest.mark.parametrize('algorithm', ['crc32', 'crc32c', 'crc64nvme'])
    def test_known_algorithm(self, algorithm):
        checksum = create_checksum_for_algorithm(algorithm)
        assert checksum is not None
        assert hasattr(checksum, 'update')
        assert hasattr(checksum, 'digest')

    def test_unknown_algorithm(self):
        assert create_checksum_for_algorithm('unknown') is None


class TestFullObjectChecksumCombiner:
    def test_combine_and_validate_crc32(self):
        data = b'hello world, this is a test of CRC combining'
        parts = [data[:15], data[15:30], data[30:]]
        expected = _compute_expected_b64(data, 'crc32')

        combiner = FullObjectChecksumCombiner(
            'crc32', len(parts), expected_b64=expected
        )
        _register_parts(combiner, parts, 'crc32')
        combiner.combine_and_validate()

    def test_combine_and_validate_crc32c(self):
        data = b'testing crc32c combining across parts'
        parts = [data[:10], data[10:]]
        expected = _compute_expected_b64(data, 'crc32c')

        combiner = FullObjectChecksumCombiner(
            'crc32c', len(parts), expected_b64=expected
        )
        _register_parts(combiner, parts, 'crc32c')
        combiner.combine_and_validate()

    def test_combine_and_validate_crc64nvme(self):
        data = b'testing crc64nvme combining across parts'
        parts = [data[:10], data[10:20], data[20:]]
        expected = _compute_expected_b64(data, 'crc64nvme')

        combiner = FullObjectChecksumCombiner(
            'crc64nvme', len(parts), expected_b64=expected
        )
        _register_parts(combiner, parts, 'crc64nvme')
        combiner.combine_and_validate()

    def test_checksum_mismatch_raises(self):
        combiner = FullObjectChecksumCombiner(
            'crc32', 1, expected_b64='AAAABB=='
        )
        checksum = create_checksum_for_algorithm('crc32')
        checksum.update(b'some data')
        combiner.register_part(0, checksum, 9)

        with pytest.raises(S3DownloadChecksumError, match='did not match'):
            combiner.combine_and_validate()

    def test_combined_b64_without_expected(self):
        data = b'upload use case'
        parts = [data[:5], data[5:]]
        expected = _compute_expected_b64(data, 'crc32')

        combiner = FullObjectChecksumCombiner('crc32', len(parts))
        _register_parts(combiner, parts, 'crc32')
        assert combiner.combined_b64 == expected

    def test_combined_bytes_are_cached(self):
        combiner = FullObjectChecksumCombiner('crc32', 1)
        checksum = create_checksum_for_algorithm('crc32')
        checksum.update(b'cache test')
        combiner.register_part(0, checksum, 10)

        assert combiner.combined_b64 == combiner.combined_b64

    def test_chunked_update_matches_single_update(self):
        data = b'streaming chunk test data for verification'
        expected = _compute_expected_b64(data, 'crc32')

        combiner = FullObjectChecksumCombiner(
            'crc32', 1, expected_b64=expected
        )
        checksum = create_checksum_for_algorithm('crc32')
        length = 0
        for i in range(0, len(data), 5):
            chunk = data[i : i + 5]
            checksum.update(chunk)
            length += len(chunk)
        combiner.register_part(0, checksum, length)
        combiner.combine_and_validate()



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import hashlib
import json

from botocore import compat

from awscli.customizations.ecs.deploy import MAX_WAIT_MIN, CodeDeployer
from awscli.customizations.ecs.exceptions import MissingPropertyError
from awscli.testutils import capture_output, mock, unittest


class TestCodeDeployer(unittest.TestCase):
    TEST_APPSPEC = {
        "version": 0.0,
        "resources": [
            {
                "TestService": {
                    "type": "AWS::ECS::Service",
                    "properties": {
                        "taskDefinition": "arn:aws:ecs:::task-definition/test:1",
                        "loadBalancerInfo": {
                            "containerName": "web",
                            "containerPort": 80,
                        },
                    },
                }
            }
        ],
    }

    def setUp(self):
        waiter = mock.Mock()
        waiter.wait.return_value = {}
        mock_cd = mock.Mock()
        mock_cd.get_waiter.return_value = waiter

        self.deployer = CodeDeployer(mock_cd, self.TEST_APPSPEC)

    def test_update_task_def_arn(self):
        test_arn = 'arn:aws:ecs::1234567890:task-definition/new-thing:3'
        self.deployer.update_task_def_arn(test_arn)

        appspec_resources = self.deployer._appspec_dict['resources']
        for resource in appspec_resources:
            actual_arn = resource['TestService']['properties'][
                'taskDefinition'
            ]
            self.assertEqual(actual_arn, test_arn)

    def test_update_task_def_arn_error_required_key(self):
        invalid_appspec = {
            "version": 0.0,
            "resources": [
                {
                    "TestFunc": {
                        "type": "AWS::Lambda::Function",
                        "properties": {"name": "some-function"},
                    }
                }
            ],
        }
        bad_deployer = CodeDeployer(None, invalid_appspec)

        with self.assertRaises(MissingPropertyError):
            bad_deployer.update_task_def_arn('arn:aws:ecs:task-definiton/test')

    def test_get_create_deploy_request(self):
        test_app = 'test-application'
        test_dgp = 'test-deployment-group'
        request = self.deployer._get_create_deploy_request(test_app, test_dgp)

        self.assertEqual(request['applicationName'], test_app)
        self.assertEqual(request['deploymentGroupName'], test_dgp)

        actual_appspec = json.loads(
            request['revision']['appSpecContent']['content']
        )
        actual_hash = request['revision']['appSpecContent']['sha256']

        self.assertEqual(actual_appspec, self.deployer._appspec_dict)
        self.assertEqual(actual_hash, self.deployer._get_appspec_hash())

    def test_get_appspec_hash(self):
        appspec_str = json.dumps(self.deployer._appspec_dict)
        encoded_appspec = compat.ensure_bytes(appspec_str)
        expected_hash = hashlib.sha256(encoded_appspec).hexdigest()

        actual_hash = self.deployer._get_appspec_hash()
        self.assertEqual(actual_hash, expected_hash)

    def test_wait_for_deploy_success_default_wait(self):
        mock_id = 'd-1234567XX'
        expected_stdout = self.deployer.MSG_WAITING.format(
            deployment_id=mock_id, wait=30
        )

        with capture_output() as captured:
            self.deployer.wait_for_deploy_success('d-1234567XX', 0)
            self.assertEqual(expected_stdout, captured.stdout.getvalue())

    def test_wait_for_deploy_success_custom_wait(self):
        mock_id = 'd-1234567XX'
        mock_wait = 40

        expected_stdout = self.deployer.MSG_WAITING.format(
            deployment_id=mock_id, wait=mock_wait
        )

        with capture_output() as captured:
            self.deployer.wait_for_deploy_success('d-1234567XX', mock_wait)
            self.assertEqual(expected_stdout, captured.stdout.getvalue())

    def test_wait_for_deploy_success_max_wait_exceeded(self):
        mock_id = 'd-1234567XX'
        mock_wait = MAX_WAIT_MIN + 15

        expected_stdout = self.deployer.MSG_WAITING.format(
            deployment_id=mock_id, wait=MAX_WAIT_MIN
        )

        with capture_output() as captured:
            self.deployer.wait_for_deploy_success('d-1234567XX', mock_wait)
            self.assertEqual(expected_stdout, captured.stdout.getvalue())



#!/usr/bin/env python
# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestDescribeInstances(BaseAWSCommandParamsTest):
    prefix = 'ec2 describe-instances'

    def test_no_params(self):
        cmdline = self.prefix
        result = {}
        self.assert_params_for_cmd(cmdline, result)

    def test_instance_id(self):
        args = ' --instance-ids i-12345678'
        cmdline = self.prefix + args
        result = {'InstanceIds': ['i-12345678']}
        self.assert_params_for_cmd(cmdline, result)

    def test_instance_ids(self):
        args = ' --instance-ids i-12345678 i-87654321'
        cmdline = self.prefix + args
        result = {'InstanceIds': ['i-12345678', 'i-87654321']}
        self.assert_params_for_cmd(cmdline, result)

    def test_instance_ids_alternate(self):
        # Not required, but will still work if you use JSON.
        args = ' --instance-ids ["i-12345678","i-87654321"]'
        cmdline = self.prefix + args
        result = {'InstanceIds': ['i-12345678', 'i-87654321']}
        self.assert_params_for_cmd(cmdline, result)

    def test_filter_json(self):
        args = """ --filters {"Name":"group-name","Values":["foobar"]}"""
        cmdline = self.prefix + args
        result = {
            'Filters': [
                {'Name': 'group-name', 'Values': ['foobar']},
            ],
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_filter_simple(self):
        args = """ --filters Name=group-name,Values=foobar"""
        cmdline = self.prefix + args
        result = {
            'Filters': [
                {'Name': 'group-name', 'Values': ['foobar']},
            ],
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_filter_values(self):
        args = """ --filters Name=group-name,Values=foobar,fiebaz"""
        cmdline = self.prefix + args
        result = {
            'Filters': [
                {'Name': 'group-name', 'Values': ['foobar', 'fiebaz']},
            ],
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_multiple_filters(self):
        args = (
            ' --filters Name=group-name,Values=foobar '
            'Name=instance-id,Values=i-12345'
        )
        cmdline = self.prefix + args
        result = {
            'Filters': [
                {'Name': 'group-name', 'Values': ['foobar']},
                {'Name': 'instance-id', 'Values': ['i-12345']},
            ],
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_multiple_filters_alternate(self):
        cmdlist = 'ec2 describe-instances'.split()
        cmdlist.extend(
            [
                '--filters',
                'Name = group-name, Values= foobar',
                'Name=instance-id,Values=i-12345',
            ]
        )
        result = {
            'Filters': [
                {'Name': 'group-name', 'Values': ['foobar']},
                {'Name': 'instance-id', 'Values': ['i-12345']},
            ],
        }
        self.assert_params_for_cmd(cmdlist, result)

    def test_page_size(self):
        args = ' --page-size 10'
        cmdline = self.prefix + args
        result = {'MaxResults': 10}
        self.assert_params_for_cmd(cmdline, result)


if __name__ == "__main__":
    unittest.main()



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os
import re
import shutil
import sys
import tempfile

from botocore.session import get_session

from awscli.customizations.eks.exceptions import EKSClusterError
from awscli.customizations.eks.kubeconfig import (
    KubeconfigCorruptedError,
    KubeconfigInaccessableError,
)
from awscli.customizations.eks.update_kubeconfig import UpdateKubeconfigCommand
from awscli.testutils import capture_output, mock, unittest
from tests.functional.eks.test_util import (
    assume_role_response,
    describe_cluster_creating_response,
    describe_cluster_response,
    get_testdata,
)


def sanitize_output(output):
    """
    Trims output and removes all lines after a line starting with warning.
    A line will only start with warning if it is the start of a
    "not installed" warning, which should be ignored when comparing output.
    """
    to_return = ""
    for line in output.splitlines():
        if bool(re.match('warning', line.strip(), re.I)):
            return to_return.strip()
        else:
            to_return += line
            to_return += '\n'
    return to_return.strip()


def build_environment(entries):
    """Build an environment variable from a list of strings."""
    return os.path.pathsep.join(entries)


class TestUpdateKubeconfig(unittest.TestCase):
    def setUp(self):
        self.create_client_patch = mock.patch(
            'botocore.session.Session.create_client'
        )

        self.mock_create_client = self.create_client_patch.start()
        self.session = get_session()

        self.client = mock.Mock()
        self.client.describe_cluster.return_value = describe_cluster_response()
        self.mock_create_client.return_value = self.client

        # Set up the sts_client_mock
        self.sts_client_mock = mock.Mock()
        self.sts_client_mock.assume_role.return_value = assume_role_response()

        # Ensure the mock_create_client correctly returns the appropriate mock
        self.mock_create_client.side_effect = lambda service_name, **kwargs: (
            self.sts_client_mock if service_name == "sts" else self.client
        )

        self.command = UpdateKubeconfigCommand(self.session)
        self.maxDiff = None

    def tearDown(self):
        self.create_client_patch.stop()

    def assert_output(self, captured, file):
        """
        Compares the captured output with the testdata named file
        For approximate equality.
        """
        with open(get_testdata(file)) as f:
            self.assertMultiLineEqual(
                sanitize_output(captured.stdout.getvalue()), f.read().strip()
            )

    def _get_temp_config(self, config):
        """
        Helper to access a temp config generated by initialize_tempfiles.
        """
        return os.path.join(self._temp_directory, config)

    def initialize_tempfiles(self, files):
        """
        Initializes a directory of tempfiles containing copies of each testdata
        file listed in files.
        Returns the absolute path of the containing directory.

        :param files: A list of filenames found in testdata
        :type files: list
        """
        self._temp_directory = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self._temp_directory)
        if files is not None:
            for file in files:
                shutil.copy2(get_testdata(file), self._get_temp_config(file))
        return self._temp_directory

    def build_temp_environment_variable(self, configs):
        """
        Generate a string which is an environment variable
        containing the paths for each temp file corresponding to configs

        :param configs: The names of the configs in testdata
        to put in the environment variable
        :type configs: list
        """
        return build_environment(
            [self._get_temp_config(config) for config in configs]
        )

    def assert_config_state(self, config_name, correct_output_name):
        """
        Asserts that the temp config named config_name has the same content
        as the testdata named correct_output_name.
        Should be called after initialize_tempfiles.

        :param config_name: The filename (not the path) of the tempfile
        to compare
        :type config_name: str

        :param correct_output_name: The filename (not the path) of the testdata
        to compare
        :type correct_output_name: str
        """
        with open(self._get_temp_config(config_name)) as file1:
            with open(get_testdata(correct_output_name)) as file2:
                self.assertMultiLineEqual(
                    file1.read().strip(), file2.read().strip()
                )

    def assert_cmd_dry(
        self,
        passed_config,
        env_variable_configs,
        default_config=os.path.join(".kube", "config"),
    ):
        """
        Run update-kubeconfig using dry-run,
        assert_cmd_dry runs directly referencing the testdata directory,
        since dry_run won't write to file
        The KUBECONFIG environment variable will be set to contain the configs
        listed in env_variable_configs (regardless of whether they exist).
        The default path will be set to default_config
        Returns the captured output

        :param passed_config: A filename to be passed to --kubeconfig
        :type passed_config: string

        :param env_variable_configs: A list of filenames to put in KUBECONFIG
        :type env_variable_configs: list or None

        :param default config: A config to be the default path
        :type default_config: string

        :returns: The captured output
        :rtype: CapturedOutput
        """
        env_variable = self.build_temp_environment_variable(
            env_variable_configs
        )
        args = ["--name", "ExampleCluster", "--dry-run"]
        if passed_config is not None:
            args += ["--kubeconfig", get_testdata(passed_config)]

        with capture_output() as captured:
            with mock.patch.dict(os.environ, {'KUBECONFIG': env_variable}):
                with mock.patch(
                    "awscli.customizations.eks.update_kubeconfig.DEFAULT_PATH",
                    get_testdata(default_config),
                ):
                    self.command(args, None)

        self.mock_create_client.assert_called_once_with('eks')
        self.client.describe_cluster.assert_called_once_with(
            name='ExampleCluster'
        )

        return captured

    def assert_cmd(
        self,
        configs,
        passed_config,
        env_variable_configs,
        default_config=os.path.join(".kube", "config"),
        verbose=False,
    ):
        """
        Run update-kubeconfig in a temp directory,
        This directory will have copies of all testdata files whose names
        are listed in configs.
        The KUBECONFIG environment variable will be set to contain the configs
        listed in env_variable_configs (regardless of whether they exist).
        The default path will be set to default_config

        :param configs: A list of filenames to copy into the temp directory
        :type configs: list

        :param passed_config: A filename to be passed to --kubeconfig
        :type passed_config: string or None

        :param env_variable_configs: A list of filenames to put in KUBECONFIG
        :type env_variable_configs: list

        :param default config: A config to be the default path
        :type default_config: string
        """
        self.initialize_tempfiles(configs)
        env_variable = self.build_temp_environment_variable(
            env_variable_configs
        )
        args = ["--name", "ExampleCluster"]
        if passed_config is not None:
            args += ["--kubeconfig", self._get_temp_config(passed_config)]
        if verbose:
            args += ["--verbose"]

        with mock.patch.dict(os.environ, {'KUBECONFIG': env_variable}):
            with mock.patch(
                "awscli.customizations.eks.update_kubeconfig.DEFAULT_PATH",
                self._get_temp_config(default_config),
            ):
                self.command(args, None)

        self.mock_create_client.assert_called_once_with('eks')
        self.client.describe_cluster.assert_called_once_with(
            name='ExampleCluster'
        )

    def test_dry_run_new(self):
        passed = "new_config"
        environment = []

        captured_output = self.assert_cmd_dry(passed, environment)
        self.assert_output(captured_output, 'output_single')

    def test_dry_run_existing(self):
        passed = "valid_existing"
        environment = []

        captured_output = self.assert_cmd_dry(passed, environment)
        self.assert_output(captured_output, 'output_combined')

    def test_dry_run_empty(self):
        passed = "valid_empty_config"
        environment = []

        captured_output = self.assert_cmd_dry(passed, environment)
        self.assert_output(captured_output, 'output_single')

    def test_dry_run_corrupted(self):
        passed = "invalid_string_clusters"
        environment = []

        with self.assertRaises(KubeconfigCorruptedError):
            captured_output = self.assert_cmd_dry(passed, environment)

    def test_write_new(self):
        configs = []
        passed = "new_config"
        environment = []

        self.assert_cmd(configs, passed, environment)
        self.assert_config_state("new_config", "output_single")

    def test_use_environment(self):
        configs = [
            'invalid_string_clusters',
            'valid_empty_existing',
            'valid_existing',
        ]
        passed = None
        environment = [
            'does_not_exist',
            'invalid_string_clusters',
            'valid_empty_existing',
            'valid_existing',
        ]

        self.assert_cmd(configs, passed, environment)
        self.assert_config_state("does_not_exist", "output_single")

    def test_use_default(self):
        configs = ["valid_existing"]
        passed = None
        environment = []
        default = "valid_existing"

        self.assert_cmd(configs, passed, environment, default, verbose=True)
        self.assert_config_state("valid_existing", "output_combined")

    def test_all_corrupted(self):
        configs = [
            "invalid_string_cluster_entry",
            "invalid_string_contexts",
            "invalid_text",
        ]
        passed = None
        environment = [
            "invalid_string_cluster_entry",
            "invalid_string_contexts",
            "invalid_text",
        ]

        with self.assertRaises(KubeconfigCorruptedError):
            self.assert_cmd(configs, passed, environment)

    def test_all_but_one_corrupted(self):
        configs = [
            "valid_existing",
            "invalid_string_cluster_entry",
            "invalid_string_contexts",
            "invalid_text",
        ]
        passed = None
        environment = [
            "valid_existing",
            "invalid_string_cluster_entry",
            "invalid_string_contexts",
            "invalid_text",
        ]

        self.assert_cmd(configs, passed, environment)
        self.assert_config_state("valid_existing", 'output_combined')

    def test_corrupted_and_missing(self):
        configs = ["invalid_string_clusters", "invalid_string_users"]
        passed = None
        environment = [
            "invalid_string_clusters",
            "does_not_exist",
            "does_not_exist2",
            "invalid_string_users",
        ]

        with self.assertRaises(KubeconfigCorruptedError):
            self.assert_cmd(configs, passed, environment)

    def test_one_corrupted_environment(self):
        configs = ["invalid_string_clusters"]
        passed = None
        environment = ["invalid_string_clusters"]

        with self.assertRaises(KubeconfigCorruptedError):
            self.assert_cmd(configs, passed, environment)

    def test_environmemt_empty_elements(self):
        configs = ["valid_existing"]

        self.initialize_tempfiles(configs)
        env_variable = build_environment(
            ["", self._get_temp_config("valid_existing")]
        )
        args = ["--name", "ExampleCluster"]

        with mock.patch.dict(os.environ, {'KUBECONFIG': env_variable}):
            with mock.patch(
                "awscli.customizations.eks.update_kubeconfig.DEFAULT_PATH",
                self._get_temp_config("default_temp"),
            ):
                self.command(args, None)

        self.mock_create_client.assert_called_once_with('eks')
        self.client.describe_cluster.assert_called_once_with(
            name='ExampleCluster'
        )
        self.assert_config_state("valid_existing", "output_combined")

    def test_environmemt_all_empty(self):
        configs = ["valid_existing"]

        self.initialize_tempfiles(configs)
        env_variable = build_environment(["", "", " ", "\t", ""])
        args = ["--name", "ExampleCluster"]

        with mock.patch.dict(os.environ, {'KUBECONFIG': env_variable}):
            with mock.patch(
                "awscli.customizations.eks.update_kubeconfig.DEFAULT_PATH",
                self._get_temp_config("default_temp"),
            ):
                self.command(args, None)

        self.mock_create_client.assert_called_once_with('eks')
        self.client.describe_cluster.assert_called_once_with(
            name='ExampleCluster'
        )
        self.assert_config_state("default_temp", "output_single")

    def test_default_path_directory(self):
        configs = []
        passed = None
        environment = []
        # Default will be the temp directory once _get_temp_config is called
        default = ""

        with self.assertRaises(KubeconfigInaccessableError):
            self.assert_cmd(configs, passed, environment, default)

    def test_update_existing(self):
        configs = ["valid_old_data"]
        passed = "valid_old_data"
        environment = []

        self.assert_cmd(configs, passed, environment)
        self.assert_config_state("valid_old_data", "output_combined")

    def test_update_existing_environment(self):
        configs = ["valid_old_data"]
        passed = None
        environment = ["valid_old_data", "output_combined", "output_single"]

        self.assert_cmd(configs, passed, environment)
        self.assert_config_state("valid_old_data", "output_combined")

    def test_cluster_creating(self):
        configs = ["output_combined"]
        passed = "output_combined"
        environment = []
        self.client.describe_cluster = mock.Mock(
            return_value=describe_cluster_creating_response()
        )
        with self.assertRaises(EKSClusterError):
            self.assert_cmd(configs, passed, environment)

    def test_kubeconfig_order(self):
        configs = ["valid_changed_ordering"]
        passed = "valid_changed_ordering"
        environment = []

        self.assert_cmd(configs, passed, environment)
        self.assert_config_state(
            "valid_changed_ordering", "output_combined_changed_ordering"
        )

    def test_update_old_api_version(self):
        configs = ["valid_old_api_version"]
        passed = "valid_old_api_version"
        environment = []

        self.assert_cmd(configs, passed, environment)
        self.assert_config_state(
            "valid_old_api_version", "valid_old_api_version_updated"
        )

    def test_assume_role(self):
        """
        Test that assume_role_arn is handled correctly when provided.
        """
        configs = ["valid_existing"]
        self.initialize_tempfiles(configs)

        # Include the --assume-role-arn argument
        args = [
            "--name",
            "ExampleCluster",
            "--assume-role-arn",
            "arn:aws:iam::123456789012:role/test-role",
        ]

        # Mock environment variables and paths
        kubeconfig_path = self._get_temp_config("valid_existing")
        default_path = self._get_temp_config("default_temp")

        with mock.patch.dict(os.environ, {'KUBECONFIG': kubeconfig_path}):
            with mock.patch(
                "awscli.customizations.eks.update_kubeconfig.DEFAULT_PATH",
                default_path,
            ):
                self.command(args, None)

        # Verify that assume_role was called with the correct parameters
        self.sts_client_mock.assume_role.assert_called_once_with(
            RoleArn="arn:aws:iam::123456789012:role/test-role",
            RoleSessionName="EKSDescribeClusterSession",
        )

        # Verify that the EKS client was created with the assumed credentials
        self.mock_create_client.assert_any_call(
            "eks",
            aws_access_key_id="test-access-key",
            aws_secret_access_key="test-secret-key",
            aws_session_token="test-session-token",
        )

        # Verify that the cluster was described
        self.client.describe_cluster.assert_called_once_with(
            name="ExampleCluster"
        )

        # Assert the configuration state
        self.assert_config_state("valid_existing", "output_combined")

    def test_no_assume_role(self):
        """
        Test that assume_role_arn is not used when not provided.
        """
        configs = ["valid_existing"]
        passed = "valid_existing"
        environment = []

        self.client.describe_cluster = mock.Mock(
            return_value=describe_cluster_response()
        )
        self.assert_cmd(configs, passed, environment)

        # Verify that assume_role was not called
        self.mock_create_client.assert_called_once_with("eks")
        self.client.describe_cluster.assert_called_once_with(
            name="ExampleCluster"
        )



class BaseBenchmarkSuite:
    """
    Base class for performance test suites. This class is used by
    the performance test runner to gather, setup, and clean up
    performance test cases. This class should be extended
    to create performance test cases that can be run by the test runner.
    """

    def __init__(self):
        self.name = self.__class__.__name__
    def get_test_cases(self, args):
        """
        Returns a list of performance test cases. Each element of the returned
        list is a generator that must generate at least args.num_iterations
        definitions for the test case.
        """
        raise NotImplementedError()

    def begin_iteration(self, case, workspace_path, assets_path, iteration):
        """
        Called before each iteration of benchmarking the specified performance
        test case. This method should handle all necessary resource creation
        and environment setup needed for the test case to execute.
        """
        pass

    def end_iteration(self, case, iteration):
        """
        Called after each iteration of benchmarking the specified performance
        test case. This method should handle all necessary teardown and cleanup
        of all resources creating in begin_iteration.
        """
        pass



from awscli.customizations.history.filters import RegexFilter
from awscli.testutils import unittest


class TestRegexFilter(unittest.TestCase):
    def assert_filter(self, pattern, replacement, input_text, expected_result):
        regex_filter = RegexFilter(pattern, replacement)
        result = regex_filter.filter_text(input_text)
        self.assertEqual(result, expected_result)

    def test_can_filter_out_content(self):
        self.assert_filter(
            pattern='foo',
            replacement='bar',
            input_text='Content with foo.',
            expected_result='Content with bar.',
        )

    def test_can_filter_with_backreferences(self):
        self.assert_filter(
            pattern='Key=(....).*',
            replacement=r'Key=\1...',
            input_text='Key=foobar',
            expected_result='Key=foob...',
        )

    def test_does_not_filter_if_no_match(self):
        self.assert_filter(
            pattern='Key=(....).*',
            replacement=r'Key=\1...',
            input_text='Blah=foobar',
            expected_result='Blah=foobar',
        )



import os
import platform
from typing import Any, List

import pytest
from build_system.install import Installer, Uninstaller

from backends.build_system.utils import Utils
from tests.markers import if_windows, skip_if_windows


class FakeUtils(Utils):
    def __init__(self, is_exe: bool, responses: bool = False):
        self._is_exe = is_exe
        self._responses = responses
        self.calls: List[Any] = []

    def isdir(self, path: str) -> bool:
        self.calls.append(("isdir", path))
        if path == os.path.join("build_dir", "exe"):
            return self._is_exe
        else:
            return self._responses

    def run(self, command, **kwargs):
        self.calls.append(("run", command, kwargs))

    def copy_directory(self, src, dst):
        self.calls.append(("copy_directory", src, dst))

    def copy_directory_contents_into(self, src, dst):
        self.calls.append(("copy_directory_contents_into", src, dst))

    def update_metadata(self, dirname, **kwargs):
        self.calls.append(("update_metadata", dirname, kwargs))

    def rmtree(self, path):
        self.calls.append(("rmtree", path))

    def read_file_lines(self, path: str) -> List[str]:
        self.calls.append(("read_file_lines", path))
        return ["file"]

    def write_file(self, path: str, content: str):
        self.calls.append(("write_file", path, content))

    def path_exists(self, path: str):
        self.calls.append(("path_exists", path))
        return self._responses

    def makedirs(self, path: str):
        self.calls.append(("makedirs", path))

    def islink(self, path: str) -> bool:
        self.calls.append(("islink", path))
        return self._responses

    def symlink(self, src: str, dst: str):
        self.calls.append(("symlink", src, dst))

    def remove(self, path: str):
        self.calls.append(("remove", path))


class TestInstaller:
    @skip_if_windows
    def test_install_exe(self):
        utils = FakeUtils(is_exe=True)
        installer = Installer("build_dir", utils=utils)
        installer.install("lib_dir", "bin_dir")

        assert utils.calls == [
            ("isdir", os.path.join("build_dir", "exe")),
            ("isdir", "lib_dir"),
            (
                "copy_directory",
                os.path.join("build_dir", "exe", "aws", "dist"),
                "lib_dir",
            ),
            ("path_exists", "bin_dir"),
            ("makedirs", "bin_dir"),
            ("islink", os.path.join("bin_dir", "aws")),
            (
                "symlink",
                os.path.join("lib_dir", "aws"),
                os.path.join("bin_dir", "aws"),
            ),
            ("islink", os.path.join("bin_dir", "aws_completer")),
            (
                "symlink",
                os.path.join("lib_dir", "aws_completer"),
                os.path.join("bin_dir", "aws_completer"),
            ),
        ]

    @if_windows
    def test_install_exe_windows(self):
        utils = FakeUtils(is_exe=True)
        installer = Installer("build_dir", utils=utils)
        installer.install("lib_dir", "bin_dir")

        assert utils.calls == [
            ("isdir", os.path.join("build_dir", "exe")),
            ("isdir", "lib_dir"),
            (
                "copy_directory",
                os.path.join("build_dir", "exe", "aws", "dist"),
                "lib_dir",
            ),
            (
                "write_file",
                "bin_dir\\aws.cmd",
                "@echo off\nlib_dir\\aws.exe %*\n",
            ),
        ]

    @skip_if_windows
    def test_install_venv(self):
        utils = FakeUtils(is_exe=False)
        installer = Installer("build_dir", utils=utils)
        installer.install("lib_dir", "bin_dir")

        assert utils.calls == [
            ("isdir", "build_dir/exe"),
            ("isdir", "lib_dir"),
            ("copy_directory", "build_dir/venv", "lib_dir"),
            ("read_file_lines", "lib_dir/bin/aws"),
            ("write_file", "lib_dir/bin/aws", "#!lib_dir/bin/python\n"),
            ("read_file_lines", "lib_dir/bin/aws_completer"),
            (
                "write_file",
                "lib_dir/bin/aws_completer",
                "#!lib_dir/bin/python\n",
            ),
            ("path_exists", "bin_dir"),
            ("makedirs", "bin_dir"),
            ("islink", "bin_dir/aws"),
            ("symlink", "lib_dir/bin/aws", "bin_dir/aws"),
            ("islink", "bin_dir/aws_completer"),
            ("symlink", "lib_dir/bin/aws_completer", "bin_dir/aws_completer"),
        ]

    @if_windows
    def test_install_venv_windows(self):
        utils = FakeUtils(is_exe=False)
        installer = Installer("build_dir", utils=utils)
        installer.install("lib_dir", "bin_dir")

        assert utils.calls == [
            ("isdir", "build_dir\\exe"),
            ("isdir", "lib_dir"),
            ("copy_directory", "build_dir\\venv", "lib_dir"),
            ("read_file_lines", "lib_dir\\Scripts\\aws.cmd"),
            (
                "write_file",
                "lib_dir\\Scripts\\aws.cmd",
                '@echo off & "lib_dir\\Scripts\\python.exe" -x "%~f0" %* & goto :eof\n',
            ),
            ("path_exists", "bin_dir"),
            ("makedirs", "bin_dir"),
            ("islink", "bin_dir\\aws.cmd"),
            ("symlink", "lib_dir\\Scripts\\aws.cmd", "bin_dir\\aws.cmd"),
        ]


class TestUninstaller:
    @skip_if_windows
    def test_uninstall(self):
        utils = FakeUtils(is_exe=True, responses=True)
        uninstaller = Uninstaller(utils=utils)
        uninstaller.uninstall("lib_dir", "bin_dir")

        assert utils.calls == [
            ("isdir", "lib_dir"),
            ("rmtree", "lib_dir"),
            ("islink", "bin_dir/aws"),
            ("remove", "bin_dir/aws"),
            ("islink", "bin_dir/aws_completer"),
            ("remove", "bin_dir/aws_completer"),
        ]

    @if_windows
    def test_uninstall_windows(self):
        utils = FakeUtils(is_exe=True, responses=True)
        uninstaller = Uninstaller(utils=utils)
        uninstaller.uninstall("lib_dir", "bin_dir")

        assert utils.calls == [
            ("isdir", "lib_dir"),
            ("rmtree", "lib_dir"),
            ("islink", "bin_dir\\aws.cmd"),
            ("remove", "bin_dir\\aws.cmd"),
        ]



#!/usr/bin/env python
# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import sys

from awscli.testutils import BaseAWSCommandParamsTest, unittest


class TestCreateTags(BaseAWSCommandParamsTest):
    prefix = 'ec2 create-tags'

    def test_create_tag_normal(self):
        cmdline = self.prefix
        cmdline += ' --resources i-12345678 --tags Key=Name,Value=bar'
        result = {
            'Resources': ['i-12345678'],
            'Tags': [{'Key': 'Name', 'Value': 'bar'}],
        }
        self.assert_params_for_cmd(cmdline, result)



# Copyright 2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from argparse import ArgumentParser

from awscli.argparser import CommandAction, FirstPassGlobalArgParser
from awscli.testutils import unittest


class TestCommandAction(unittest.TestCase):
    def setUp(self):
        self.parser = ArgumentParser()

    def test_choices(self):
        command_table = {'pre-existing': object()}
        self.parser.add_argument(
            'command', action=CommandAction, command_table=command_table
        )
        parsed_args = self.parser.parse_args(['pre-existing'])
        self.assertEqual(parsed_args.command, 'pre-existing')

    def test_choices_added_after(self):
        command_table = {'pre-existing': object()}
        self.parser.add_argument(
            'command', action=CommandAction, command_table=command_table
        )
        command_table['after'] = object()

        # The pre-existing command should still be able to be parsed
        parsed_args = self.parser.parse_args(['pre-existing'])
        self.assertEqual(parsed_args.command, 'pre-existing')

        # The command added after the argument's creation should be
        # able to be parsed as well.
        parsed_args = self.parser.parse_args(['after'])
        self.assertEqual(parsed_args.command, 'after')


class TestPartialGlobalArgsParser(unittest.TestCase):
    def setUp(self):
        self.parser = FirstPassGlobalArgParser()

    def test_can_parse_profile(self):
        parsed_args, _ = self.parser.parse_known_args(['--profile', 'foo'])
        self.assertEqual(parsed_args.profile, 'foo')

    def test_can_parse_debug(self):
        parsed_args, _ = self.parser.parse_known_args(['--debug'])
        self.assertTrue(parsed_args.debug)

    def test_debug_if_false_by_default(self):
        parsed_args, _ = self.parser.parse_known_args(['--foo'])
        self.assertFalse(parsed_args.debug)

    def test_not_parse_unknown_args(self):
        parsed_args, remains = self.parser.parse_known_args(
            ['--debug', '--foo', 'bar']
        )
        self.assertEqual(parsed_args.debug, True)
        self.assertEqual(remains, ['--foo', 'bar'])



#!/usr/bin/env python
# Copyright 2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestConfigureHealthCheck(BaseAWSCommandParamsTest):
    prefix = 'elb configure-health-check'

    def test_shorthand_basic(self):
        cmdline = self.prefix
        cmdline += ' --load-balancer-name my-lb'
        cmdline += (
            ' --health-check Target=HTTP:80/weather/us/wa/seattle,'
            'Interval=300,Timeout=60,UnhealthyThreshold=5,'
            'HealthyThreshold=9'
        )
        result = {
            'HealthCheck': {
                'HealthyThreshold': 9,
                'Interval': 300,
                'Target': 'HTTP:80/weather/us/wa/seattle',
                'Timeout': 60,
                'UnhealthyThreshold': 5,
            },
            'LoadBalancerName': 'my-lb',
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_json(self):
        cmdline = self.prefix
        cmdline += ' --load-balancer-name my-lb '
        cmdline += (
            '--health-check {"Target":"HTTP:80/weather/us/wa/seattle'
            '?a=b","Interval":300,"Timeout":60,'
            '"UnhealthyThreshold":5,"HealthyThreshold":9}'
        )
        result = {
            'HealthCheck': {
                'HealthyThreshold': 9,
                'Interval': 300,
                'Target': 'HTTP:80/weather/us/wa/seattle?a=b',
                'Timeout': 60,
                'UnhealthyThreshold': 5,
            },
            'LoadBalancerName': 'my-lb',
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_shorthand_with_multiple_equals_for_value(self):
        cmdline = self.prefix
        cmdline += ' --load-balancer-name my-lb'
        cmdline += (
            ' --health-check Target="HTTP:80/weather/us/wa/seattle?a=b"'
            ',Interval=300,Timeout=60,UnhealthyThreshold=5,'
            'HealthyThreshold=9'
        )
        result = {
            'HealthCheck': {
                'HealthyThreshold': 9,
                'Interval': 300,
                'Target': 'HTTP:80/weather/us/wa/seattle?a=b',
                'Timeout': 60,
                'UnhealthyThreshold': 5,
            },
            'LoadBalancerName': 'my-lb',
        }
        self.assert_params_for_cmd(cmdline, result)



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest, mock


class TestCreateInvalidation(BaseAWSCommandParamsTest):
    prefix = 'cloudfront create-invalidation --distribution-id my_id '

    def test_invalidation_batch_only(self):
        batch = "Paths={Quantity=2,Items=[foo.txt,bar.txt]},CallerReference=ab"
        cmdline = self.prefix + '--invalidation-batch ' + batch
        result = {
            'DistributionId': 'my_id',
            'InvalidationBatch': {
                'Paths': {'Items': ['foo.txt', 'bar.txt'], 'Quantity': 2},
                'CallerReference': 'ab',
            },
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_paths_only(self):
        cmdline = self.prefix + '--paths index.html foo.txt'
        result = {
            'DistributionId': 'my_id',
            'InvalidationBatch': {
                'Paths': {'Items': ['index.html', 'foo.txt'], 'Quantity': 2},
                'CallerReference': mock.ANY,
            },
        }
        self.run_cmd(cmdline)
        self.assertEqual(self.last_kwargs, result)

    def test_invalidation_batch_and_paths(self):
        cmdline = self.prefix + '--invalidation-batch {} --paths foo'
        self.run_cmd(cmdline, expected_rc=252)

    def test_neither_invalidation_batch_or_paths(self):
        self.run_cmd(self.prefix, expected_rc=252)



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import pytest
from botocore.model import OperationModel

from awscli.clidriver import create_clidriver

# Excluded commands must be registered in awscli/customizations/removals.py
_ALLOWED_COMMANDS = ['s3api select-object-content']


def _generate_command_tests():
    driver = create_clidriver()
    help_command = driver.create_help_command()
    for command_name, command_obj in help_command.command_table.items():
        sub_help = command_obj.create_help_command()
        if hasattr(sub_help, 'command_table'):
            for sub_name, sub_command in sub_help.command_table.items():
                op_help = sub_command.create_help_command()
                model = op_help.obj
                if isinstance(model, OperationModel):
                    # Extract the properties needed for tests to avoid
                    # parametrizing entire model objects, which may cause
                    # excessive memory usage.
                    if not model.has_event_stream_input and not model.has_event_stream_output:
                        # Only parameterize the models that actually have event streams.
                        # This has shown to improve test execution performance.
                        continue
                    model_description = {
                        'service_name': model.service_model.service_name,
                        'name': model.name,
                    }
                    yield command_name, sub_name, model_description


@pytest.mark.validates_models
@pytest.mark.parametrize(
    "command_name, sub_name, model", _generate_command_tests()
)
def test_no_event_stream_unless_allowed(
        command_name,
        sub_name,
        model,
        record_property
):
    full_command = f'{command_name} {sub_name}'
    # Store the service and operation in
    # PyTest custom properties
    record_property(
        'aws_service', model['service_name']
    )
    record_property('aws_operation', model['name'])
    supported_commands = '\n'.join(_ALLOWED_COMMANDS)
    assert full_command in _ALLOWED_COMMANDS, (
        f'The {full_command} command uses event streams '
        'which is only supported for these operations:\n'
        f'{supported_commands}'
    )



# Copyright 2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.customizations.dynamodb.paginatorfix import (
    parse_last_evaluated_key_binary,
)
from awscli.testutils import unittest


class TestParseLastEvaluatedKeyBinary(unittest.TestCase):
    def assert_parsed_output(self, last_key_value, expected_output):
        parsed = {
            'LastEvaluatedKey': {
                'FooKeyName': last_key_value,
            }
        }
        parse_last_evaluated_key_binary(parsed=parsed)
        self.assertEqual(last_key_value, expected_output)

    def test_ignores_response_without_last_evaluated_key(self):
        parsed = {'Items': []}
        parse_last_evaluated_key_binary(parsed=parsed)
        self.assertEqual(parsed, {'Items': []})

    def test_parses_binary_key(self):
        self.assert_parsed_output({'B': 'Zm9v'}, {'B': b'foo'})

    def test_ignores_strings(self):
        self.assert_parsed_output({'S': 'Zm9v'}, {'S': 'Zm9v'})

    def test_ignores_numbers(self):
        self.assert_parsed_output({'N': 2}, {'N': 2})



# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import functools
import os

from constants import (
    BIN_DIRNAME,
    CLI_SCRIPTS,
    IS_WINDOWS,
    PYTHON_EXE_NAME,
    ArtifactType,
)
from utils import Utils

WINDOWS_CMD_TEMPLATE = """@echo off
{path} %*
"""


class Uninstaller:
    def __init__(self, utils: Utils = None):
        if utils is None:
            utils = Utils()
        self._utils = utils

    def uninstall(self, install_dir: str, bin_dir: str):
        if self._utils.isdir(install_dir):
            self._utils.rmtree(install_dir)
        for exe in CLI_SCRIPTS:
            exe_path = os.path.join(bin_dir, exe)
            if self._utils.islink(exe_path) or self._utils.path_exists(
                exe_path
            ):
                self._utils.remove(exe_path)


class Installer:
    def __init__(self, build_dir: str, utils: Utils = None):
        self._build_dir = build_dir
        if utils is None:
            utils = Utils()
        self._utils = utils

    def install(self, install_dir: str, bin_dir: str):
        self._copy_to_install_dir(install_dir)
        self._install_executables(install_dir, bin_dir)

    @functools.cached_property
    def artifact_type(self):
        if self._utils.isdir(os.path.join(self._build_dir, "exe")):
            return ArtifactType.PORTABLE_EXE
        return ArtifactType.SYSTEM_SANDBOX

    def _copy_to_install_dir(self, install_dir):
        build_lib = self._get_build_lib()
        if self._utils.isdir(install_dir):
            self._utils.rmtree(install_dir)
        self._utils.copy_directory(build_lib, install_dir)
        if self.artifact_type == ArtifactType.SYSTEM_SANDBOX:
            self._update_script_header(install_dir)

    def _get_build_lib(self):
        if self.artifact_type == ArtifactType.PORTABLE_EXE:
            return os.path.join(self._build_dir, "exe", "aws", "dist")
        return os.path.join(self._build_dir, "venv")

    def _install_executables(self, install_dir, bin_dir):
        if IS_WINDOWS and self.artifact_type == ArtifactType.PORTABLE_EXE:
            self._install_executables_on_windows(install_dir, bin_dir)
        else:
            self._symlink_executables(install_dir, bin_dir)

    def _install_executables_on_windows(self, install_dir, bin_dir):
        filepath = os.path.join(bin_dir, "aws.cmd")
        content = WINDOWS_CMD_TEMPLATE.format(
            path=os.path.join(install_dir, "aws.exe")
        )
        self._utils.write_file(filepath, content)

    def _symlink_executables(self, install_dir, bin_dir):
        if not self._utils.path_exists(bin_dir):
            self._utils.makedirs(bin_dir)
        for exe in CLI_SCRIPTS:
            exe_path = os.path.join(bin_dir, exe)
            if self._utils.islink(exe_path):
                self._utils.remove(exe_path)
            self._utils.symlink(
                self._get_install_bin_exe(install_dir, exe), exe_path
            )

    def _get_install_bin_exe(self, install_dir, exe):
        install_bin_dir = install_dir
        if self.artifact_type == ArtifactType.SYSTEM_SANDBOX:
            install_bin_dir = os.path.join(install_dir, BIN_DIRNAME)
        return os.path.join(install_bin_dir, exe)

    def _update_script_header(self, install_dir):
        python_exe_path = self._get_install_bin_exe(
            install_dir, PYTHON_EXE_NAME
        )
        for exe in CLI_SCRIPTS:
            exe_path = self._get_install_bin_exe(install_dir, exe)
            lines = self._utils.read_file_lines(exe_path)
            lines[0] = self._utils.get_script_header(python_exe_path)
            self._utils.write_file(exe_path, "".join(lines))



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.compat import StringIO
from awscli.customizations.configure.listprofiles import ListProfilesCommand
from awscli.testutils import unittest

from . import FakeSession


class TestListProfilesCommand(unittest.TestCase):
    def _create_command(self, profiles=None):
        session = FakeSession({}, available_profiles=profiles)
        stdout = StringIO()
        command = ListProfilesCommand(session, out_stream=stdout)
        return stdout, command

    def test_lists_profile_default(self):
        stdout, list_profiles = self._create_command()
        list_profiles(args=[], parsed_globals=None)
        self.assertEqual('default\n', stdout.getvalue())

    def test_lists_profile_multiple(self):
        profiles = [
            'default',
            'profile-foo',
            'profile-baz',
            'some profile',
        ]
        stdout, list_profiles = self._create_command(profiles=profiles)
        list_profiles(args=[], parsed_globals=None)
        self.assertEqual('\n'.join(profiles) + '\n', stdout.getvalue())
